# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# datadog
from datadog.api.resources import GetableAPIResource, UpdatableAPIResource


class Metadata(GetableAPIResource, UpdatableAPIResource):
    """
    A wrapper around Metric Metadata HTTP API
    """

    _resource_name = "metrics"

    @classmethod
    def get(cls, metric_name):
        """
        Get metadata information on an existing Datadog metric

        param metric_name: metric name (ex. system.cpu.idle)

        :returns: Dictionary representing the API's JSON response
        """
        if not metric_name:
            raise KeyError("'metric_name' parameter is required")

        return super(Metadata, cls).get(metric_name)

    @classmethod
    def update(cls, metric_name, **params):
        """
        Update metadata fields for an existing Datadog metric.
        If the metadata does not exist for the metric it is created by
        the update.

        :param type: type of metric (ex. "gauge", "rate", etc.)
                            see http://docs.datadoghq.com/metrictypes/
        :type type: string

        :param description: description of the metric
        :type description: string

        :param short_name: short name of the metric
        :type short_name: string

        :param unit: unit type associated with the metric (ex. "byte", "operation")
                     see http://docs.datadoghq.com/units/ for full list
        :type unit: string

        :param per_unit: per unit type (ex. "second" as in "queries per second")
                         see http://docs.datadoghq.com/units/ for full list
        :type per_unit: string

        :param statsd_interval: statsd flush interval for metric in seconds (if applicable)
        :type statsd_interval: integer

        :returns: Dictionary representing the API's JSON response

        >>> api.Metadata.update(metric_name='api.requests.served', metric_type="counter")
        """
        if not metric_name:
            raise KeyError("'metric_name' parameter is required")

        return super(Metadata, cls).update(id=metric_name, **params)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
from __future__ import print_function
import os
import sys

# datadog
from datadog.util.compat import is_p3k, configparser, IterableUserDict, get_input


def print_err(msg):
    if is_p3k():
        print(msg + "\n", file=sys.stderr)
    else:
        sys.stderr.write(msg + "\n")
    sys.stderr.flush()


def report_errors(res):
    if "errors" in res:
        errors = res["errors"]
        if isinstance(errors, list):
            for error in errors:
                print_err("ERROR: {}".format(error))
        else:
            print_err("ERROR: {}".format(errors))
        sys.exit(1)
    return False


def report_warnings(res):
    if "warnings" in res:
        warnings = res["warnings"]
        if isinstance(warnings, list):
            for warning in warnings:
                print_err("WARNING: {}".format(warning))
        else:
            print_err("WARNING: {}".format(warnings))
        return True
    return False


class DogshellConfig(IterableUserDict):
    def load(self, config_file, api_key, app_key):
        config = configparser.ConfigParser()

        if api_key is not None and app_key is not None:
            self["api_key"] = api_key
            self["app_key"] = app_key
        else:
            if os.access(config_file, os.F_OK):
                config.read(config_file)
                if not config.has_section("Connection"):
                    report_errors({"errors": ["%s has no [Connection] section" % config_file]})
            else:
                try:
                    response = None
                    while response is None or response.strip().lower() not in ["", "y", "n"]:
                        response = get_input("%s does not exist. Would you like to" " create it? [Y/n] " % config_file)
                        if response.strip().lower() in ["", "y"]:
                            # Read the api and app keys from stdin
                            api_key = get_input(
                                "What is your api key? (Get it here: "
                                "https://app.datadoghq.com/account/settings#api) "
                            )
                            app_key = get_input(
                                "What is your application key? (Generate one here: "
                                "https://app.datadoghq.com/account/settings#api) "
                            )

                            # Write the config file
                            config.add_section("Connection")
                            config.set("Connection", "apikey", api_key)
                            config.set("Connection", "appkey", app_key)

                            f = open(config_file, "w")
                            config.write(f)
                            f.close()
                            print("Wrote %s" % config_file)
                        elif response.strip().lower() == "n":
                            # Abort
                            print_err("Exiting\n")
                            sys.exit(1)
                except KeyboardInterrupt:
                    # Abort
                    print_err("\nExiting")
                    sys.exit(1)

            self["api_key"] = config.get("Connection", "apikey")
            self["app_key"] = config.get("Connection", "appkey")
            if config.has_section("Proxy"):
                self["proxies"] = dict(config.items("Proxy"))
            if config.has_option("Connection", "host_name"):
                self["host_name"] = config.get("Connection", "host_name")
            if config.has_option("Connection", "api_host"):
                self["api_host"] = config.get("Connection", "api_host")
        assert self["api_key"] is not None and self["app_key"] is not None



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import json
import logging
import re
import socket
import subprocess
import types
from typing import Dict, Optional

# datadog
from datadog.util.compat import url_lib, is_p3k, iteritems
from datadog.util.config import get_config, get_os, CfgNotFound

VALID_HOSTNAME_RFC_1123_PATTERN = re.compile(
    r"^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$"
)  # noqa
MAX_HOSTNAME_LEN = 255

log = logging.getLogger("datadog.api")


def is_valid_hostname(hostname):
    if hostname.lower() in set(
        [
            "localhost",
            "localhost.localdomain",
            "localhost6.localdomain6",
            "ip6-localhost",
        ]
    ):
        log.warning("Hostname: %s is local" % hostname)
        return False
    if len(hostname) > MAX_HOSTNAME_LEN:
        log.warning("Hostname: %s is too long (max length is  %s characters)" % (hostname, MAX_HOSTNAME_LEN))
        return False
    if VALID_HOSTNAME_RFC_1123_PATTERN.match(hostname) is None:
        log.warning("Hostname: %s is not complying with RFC 1123" % hostname)
        return False
    return True


def get_hostname(hostname_from_config):
    # type: (bool) -> Optional[str]
    """
    Get the canonical host name this agent should identify as. This is
    the authoritative source of the host name for the agent.

    Tries, in order:

      * agent config (datadog.conf, "hostname:")
      * 'hostname -f' (on unix)
      * socket.gethostname()
    """

    hostname = None
    config = None

    # first, try the config if hostname_from_config is set to True
    try:
        if hostname_from_config:
            config = get_config()
            config_hostname = config.get("hostname")
            if config_hostname and is_valid_hostname(config_hostname):
                log.warning(
                    "Hostname lookup from agent configuration will be deprecated "
                    "in an upcoming version of datadogpy. Set hostname_from_config to False "
                    "to get rid of this warning"
                )
                return config_hostname
    except CfgNotFound:
        log.info("No agent or invalid configuration file found")

    # Try to get GCE instance name
    if hostname is None:
        gce_hostname = GCE.get_hostname(config)
        if gce_hostname is not None:
            if is_valid_hostname(gce_hostname):
                return gce_hostname
    # then move on to os-specific detection
    if hostname is None:

        def _get_hostname_unix():
            try:
                # try fqdn
                p = subprocess.Popen(["/bin/hostname", "-f"], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
                out, err = p.communicate()
                if p.returncode == 0:
                    if is_p3k():
                        return out.decode("utf-8").strip()
                    else:
                        return out.strip()
            except Exception:
                return None

        os_name = get_os()
        if os_name in ["mac", "freebsd", "linux", "solaris"]:
            unix_hostname = _get_hostname_unix()
            if unix_hostname and is_valid_hostname(unix_hostname):
                hostname = unix_hostname

    # if we have an ec2 default hostname, see if there's an instance-id available
    if hostname is not None and True in [hostname.lower().startswith(p) for p in [u"ip-", u"domu"]]:
        instanceid = EC2.get_instance_id(config)
        if instanceid:
            hostname = instanceid

    # fall back on socket.gethostname(), socket.getfqdn() is too unreliable
    if hostname is None:
        try:
            socket_hostname = socket.gethostname()  # type: Optional[str]
        except socket.error:
            socket_hostname = None
        if socket_hostname and is_valid_hostname(socket_hostname):
            hostname = socket_hostname

    if hostname is None:
        log.warning(
            u"Unable to reliably determine host name. You can define one in your `hosts` file, "
            u"or in `datadog.conf` file if you have Datadog Agent installed."
        )

    return hostname


def get_ec2_instance_id():
    try:
        # Remember the previous default timeout
        old_timeout = socket.getdefaulttimeout()

        # Try to query the EC2 internal metadata service, but fail fast
        socket.setdefaulttimeout(0.25)

        try:
            return url_lib.urlopen(url_lib.Request("http://169.254.169.254/latest/" "meta-data/instance-id")).read()
        finally:
            # Reset the previous default timeout
            socket.setdefaulttimeout(old_timeout)
    except Exception:
        return socket.gethostname()


class GCE(object):
    URL = "http://169.254.169.254/computeMetadata/v1/?recursive=true"
    TIMEOUT = 0.1  # second
    SOURCE_TYPE_NAME = "google cloud platform"
    metadata = None

    @staticmethod
    def _get_metadata(agentConfig):
        if GCE.metadata is not None:
            return GCE.metadata

        if not agentConfig["collect_instance_metadata"]:
            log.info("Instance metadata collection is disabled. Not collecting it.")
            GCE.metadata = {}
            return GCE.metadata

        socket_to = None
        try:
            socket_to = socket.getdefaulttimeout()
            socket.setdefaulttimeout(GCE.TIMEOUT)
        except Exception:
            pass

        try:
            opener = url_lib.build_opener()
            opener.addheaders = [("X-Google-Metadata-Request", "True")]
            GCE.metadata = json.loads(opener.open(GCE.URL).read().strip())

        except Exception:
            GCE.metadata = {}

        try:
            if socket_to is None:
                socket_to = 3
            socket.setdefaulttimeout(socket_to)
        except Exception:
            pass
        return GCE.metadata

    @staticmethod
    def get_hostname(agentConfig):
        try:
            host_metadata = GCE._get_metadata(agentConfig)
            return host_metadata["instance"]["hostname"].split(".")[0]
        except Exception:
            return None


class EC2(object):
    """Retrieve EC2 metadata"""

    URL = "http://169.254.169.254/latest/meta-data"
    TIMEOUT = 0.1  # second
    metadata = {}  # type: Dict[str, str]

    @staticmethod
    def get_tags(agentConfig):
        if not agentConfig["collect_instance_metadata"]:
            log.info("Instance metadata collection is disabled. Not collecting it.")
            return []

        socket_to = None
        try:
            socket_to = socket.getdefaulttimeout()
            socket.setdefaulttimeout(EC2.TIMEOUT)
        except Exception:
            pass

        try:
            iam_role = url_lib.urlopen(EC2.URL + "/iam/security-credentials").read().strip()
            iam_params = json.loads(
                url_lib.urlopen(EC2.URL + "/iam/security-credentials" + "/" + str(iam_role)).read().strip()
            )
            from boto.ec2.connection import EC2Connection

            connection = EC2Connection(
                aws_access_key_id=iam_params["AccessKeyId"],
                aws_secret_access_key=iam_params["SecretAccessKey"],
                security_token=iam_params["Token"],
            )
            instance_object = connection.get_only_instances([EC2.metadata["instance-id"]])[0]

            EC2_tags = [u"%s:%s" % (tag_key, tag_value) for tag_key, tag_value in iteritems(instance_object.tags)]

        except Exception:
            log.exception("Problem retrieving custom EC2 tags")
            EC2_tags = []

        try:
            if socket_to is None:
                socket_to = 3
            socket.setdefaulttimeout(socket_to)
        except Exception:
            pass

        return EC2_tags

    @staticmethod
    def get_metadata(agentConfig):
        """Use the ec2 http service to introspect the instance. This adds latency \
        if not running on EC2
        """
        # >>> import urllib2
        # >>> urllib2.urlopen('http://169.254.169.254/latest/', timeout=1).read()
        # 'meta-data\nuser-data'
        # >>> urllib2.urlopen('http://169.254.169.254/latest/meta-data', timeout=1).read()
        # 'ami-id\nami-launch-index\nami-manifest-path\nhostname\ninstance-id\nlocal-ipv4\
        # npublic-keys/\nreservation-id\nsecurity-groups'
        # >>> urllib2.urlopen('http://169.254.169.254/latest/meta-data/instance-id',
        # timeout=1).read()
        # 'i-deadbeef'

        # Every call may add TIMEOUT seconds in latency so don't abuse this call
        # python 2.4 does not support an explicit timeout argument so force it here
        # Rather than monkey-patching urllib2, just lower the timeout globally for these calls

        if not agentConfig["collect_instance_metadata"]:
            log.info("Instance metadata collection is disabled. Not collecting it.")
            return {}

        socket_to = None
        try:
            socket_to = socket.getdefaulttimeout()
            socket.setdefaulttimeout(EC2.TIMEOUT)
        except Exception:
            pass

        for k in (
            "instance-id",
            "hostname",
            "local-hostname",
            "public-hostname",
            "ami-id",
            "local-ipv4",
            "public-keys",
            "public-ipv4",
            "reservation-id",
            "security-groups",
        ):
            try:
                v = url_lib.urlopen(EC2.URL + "/" + str(k)).read().strip()
                assert type(v) in (types.StringType, types.UnicodeType) and len(v) > 0, "%s is not a string" % v
                EC2.metadata[k] = v
            except Exception:
                pass

        try:
            if socket_to is None:
                socket_to = 3
            socket.setdefaulttimeout(socket_to)
        except Exception:
            pass

        return EC2.metadata

    @staticmethod
    def get_instance_id(agentConfig):
        try:
            return EC2.get_metadata(agentConfig).get("instance-id", None)
        except Exception:
            return None



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import json
import os.path
import platform
import sys
import webbrowser

# 3p
import argparse

# datadog
from datadog import api
from datadog.dogshell.common import report_errors, report_warnings, print_err
from datadog.util.format import pretty_json
from datetime import datetime


class TimeboardClient(object):
    @classmethod
    def setup_parser(cls, subparsers):
        parser = subparsers.add_parser("timeboard", help="Create, edit, and delete timeboards")
        parser.add_argument(
            "--string_ids",
            action="store_true",
            dest="string_ids",
            help="Represent timeboard IDs as strings instead of ints in JSON",
        )

        verb_parsers = parser.add_subparsers(title="Verbs", dest="verb")
        verb_parsers.required = True

        post_parser = verb_parsers.add_parser("post", help="Create timeboards")
        post_parser.add_argument("title", help="title for the new timeboard")
        post_parser.add_argument("description", help="short description of the timeboard")
        post_parser.add_argument(
            "graphs", help="graph definitions as a JSON string. if unset," " reads from stdin.", nargs="?"
        )
        post_parser.add_argument(
            "--template_variables",
            type=_template_variables,
            default=[],
            help="a json list of template variable dicts, e.g. "
            "[{'name': 'host', 'prefix': 'host', "
            "'default': 'host:my-host'}]'",
        )

        post_parser.set_defaults(func=cls._post)

        update_parser = verb_parsers.add_parser("update", help="Update existing timeboards")
        update_parser.add_argument("timeboard_id", help="timeboard to replace" " with the new definition")
        update_parser.add_argument("title", help="new title for the timeboard")
        update_parser.add_argument("description", help="short description of the timeboard")
        update_parser.add_argument(
            "graphs", help="graph definitions as a JSON string." " if unset, reads from stdin", nargs="?"
        )
        update_parser.add_argument(
            "--template_variables",
            type=_template_variables,
            default=[],
            help="a json list of template variable dicts, e.g. "
            "[{'name': 'host', 'prefix': 'host', "
            "'default': 'host:my-host'}]'",
        )
        update_parser.set_defaults(func=cls._update)

        show_parser = verb_parsers.add_parser("show", help="Show a timeboard definition")
        show_parser.add_argument("timeboard_id", help="timeboard to show")
        show_parser.set_defaults(func=cls._show)

        show_all_parser = verb_parsers.add_parser("show_all", help="Show a list of all timeboards")
        show_all_parser.set_defaults(func=cls._show_all)

        pull_parser = verb_parsers.add_parser("pull", help="Pull a timeboard on the server" " into a local file")
        pull_parser.add_argument("timeboard_id", help="ID of timeboard to pull")
        pull_parser.add_argument("filename", help="file to pull timeboard into")
        pull_parser.set_defaults(func=cls._pull)

        pull_all_parser = verb_parsers.add_parser("pull_all", help="Pull all timeboards" " into files in a directory")
        pull_all_parser.add_argument("pull_dir", help="directory to pull timeboards into")
        pull_all_parser.set_defaults(func=cls._pull_all)

        push_parser = verb_parsers.add_parser(
            "push", help="Push updates to timeboards" " from local files to the server"
        )
        push_parser.add_argument(
            "--append_auto_text",
            action="store_true",
            dest="append_auto_text",
            help="When pushing to the server, appends filename"
            " and timestamp to the end of the timeboard description",
        )
        push_parser.add_argument(
            "file", help="timeboard files to push to the server", nargs="+", type=argparse.FileType("r")
        )
        push_parser.set_defaults(func=cls._push)

        new_file_parser = verb_parsers.add_parser(
            "new_file", help="Create a new timeboard" " and put its contents in a file"
        )
        new_file_parser.add_argument("filename", help="name of file to create with empty timeboard")
        new_file_parser.add_argument(
            "graphs", help="graph definitions as a JSON string." " if unset, reads from stdin.", nargs="?"
        )
        new_file_parser.set_defaults(func=cls._new_file)

        web_view_parser = verb_parsers.add_parser("web_view", help="View the timeboard in a web browser")
        web_view_parser.add_argument("file", help="timeboard file", type=argparse.FileType("r"))
        web_view_parser.set_defaults(func=cls._web_view)

        delete_parser = verb_parsers.add_parser("delete", help="Delete timeboards")
        delete_parser.add_argument("timeboard_id", help="timeboard to delete")
        delete_parser.set_defaults(func=cls._delete)

    @classmethod
    def _pull(cls, args):
        cls._write_dash_to_file(args.timeboard_id, args.filename, args.timeout, args.format, args.string_ids)

    @classmethod
    def _pull_all(cls, args):
        api._timeout = args.timeout

        def _title_to_filename(title):
            # Get a lowercased version with most punctuation stripped out...
            no_punct = "".join([c for c in title.lower() if c.isalnum() or c in [" ", "_", "-"]])
            # Now replace all -'s, _'s and spaces with "_", and strip trailing _
            return no_punct.replace(" ", "_").replace("-", "_").strip("_")

        format = args.format
        res = api.Timeboard.get_all()
        report_warnings(res)
        report_errors(res)

        if not os.path.exists(args.pull_dir):
            os.mkdir(args.pull_dir, 0o755)

        used_filenames = set()
        for dash_summary in res["dashes"]:
            filename = _title_to_filename(dash_summary["title"])
            if filename in used_filenames:
                filename = filename + "-" + dash_summary["id"]
            used_filenames.add(filename)

            cls._write_dash_to_file(
                dash_summary["id"],
                os.path.join(args.pull_dir, filename + ".json"),
                args.timeout,
                format,
                args.string_ids,
            )
        if format == "pretty":
            print(
                ("\n### Total: {0} dashboards to {1} ###".format(len(used_filenames), os.path.realpath(args.pull_dir)))
            )

    @classmethod
    def _new_file(cls, args):
        api._timeout = args.timeout
        format = args.format
        graphs = args.graphs
        if args.graphs is None:
            graphs = sys.stdin.read()
        graphs = json.loads(graphs)
        res = api.Timeboard.create(
            title=args.filename, description="Description for {0}".format(args.filename), graphs=[graphs]
        )

        report_warnings(res)
        report_errors(res)

        cls._write_dash_to_file(res["dash"]["id"], args.filename, args.timeout, format, args.string_ids)

        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _write_dash_to_file(cls, dash_id, filename, timeout, format="raw", string_ids=False):
        with open(filename, "w") as f:
            res = api.Timeboard.get(dash_id)
            report_warnings(res)
            report_errors(res)

            dash_obj = res["dash"]
            if "resource" in dash_obj:
                del dash_obj["resource"]
            if "url" in dash_obj:
                del dash_obj["url"]

            if string_ids:
                dash_obj["id"] = str(dash_obj["id"])

            if not dash_obj.get("template_variables"):
                dash_obj.pop("template_variables", None)

            json.dump(dash_obj, f, indent=2)

            if format == "pretty":
                print(u"Downloaded dashboard {0} to file {1}".format(dash_id, filename))
            else:
                print(u"{0} {1}".format(dash_id, filename))

    @classmethod
    def _push(cls, args):
        api._timeout = args.timeout
        for f in args.file:
            try:
                dash_obj = json.load(f)
            except Exception as err:
                raise Exception("Could not parse {0}: {1}".format(f.name, err))

            if args.append_auto_text:
                datetime_str = datetime.now().strftime("%x %X")
                auto_text = "<br/>\nUpdated at {0} from {1} ({2}) on {3}".format(
                    datetime_str, f.name, dash_obj["id"], platform.node()
                )
                dash_obj["description"] += auto_text
            tpl_vars = dash_obj.get("template_variables", [])

            if "id" in dash_obj:
                # Always convert to int, in case it was originally a string.
                dash_obj["id"] = int(dash_obj["id"])
                res = api.Timeboard.update(
                    dash_obj["id"],
                    title=dash_obj["title"],
                    description=dash_obj["description"],
                    graphs=dash_obj["graphs"],
                    template_variables=tpl_vars,
                )
            else:
                res = api.Timeboard.create(
                    title=dash_obj["title"],
                    description=dash_obj["description"],
                    graphs=dash_obj["graphs"],
                    template_variables=tpl_vars,
                )

            if "errors" in res:
                print_err("Upload of dashboard {0} from file {1} failed.".format(dash_obj["id"], f.name))

            report_warnings(res)
            report_errors(res)

            if format == "pretty":
                print(pretty_json(res))
            else:
                print(json.dumps(res))

            if args.format == "pretty":
                print("Uploaded file {0} (dashboard {1})".format(f.name, dash_obj["id"]))

    @classmethod
    def _post(cls, args):
        api._timeout = args.timeout
        format = args.format
        graphs = args.graphs
        if args.graphs is None:
            graphs = sys.stdin.read()
        graphs = json.loads(graphs)
        res = api.Timeboard.create(
            title=args.title, description=args.description, graphs=[graphs], template_variables=args.template_variables
        )
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _update(cls, args):
        api._timeout = args.timeout
        format = args.format
        graphs = args.graphs
        if args.graphs is None:
            graphs = sys.stdin.read()
        graphs = json.loads(graphs)

        res = api.Timeboard.update(
            args.timeboard_id,
            title=args.title,
            description=args.description,
            graphs=graphs,
            template_variables=args.template_variables,
        )
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _show(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Timeboard.get(args.timeboard_id)
        report_warnings(res)
        report_errors(res)

        if args.string_ids:
            res["dash"]["id"] = str(res["dash"]["id"])

        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _show_all(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Timeboard.get_all()
        report_warnings(res)
        report_errors(res)

        if args.string_ids:
            for d in res["dashes"]:
                d["id"] = str(d["id"])

        if format == "pretty":
            print(pretty_json(res))
        elif format == "raw":
            print(json.dumps(res))
        else:
            for d in res["dashes"]:
                print("\t".join([(d["id"]), (d["resource"]), (d["title"]), cls._escape(d["description"])]))

    @classmethod
    def _delete(cls, args):
        api._timeout = args.timeout
        res = api.Timeboard.delete(args.timeboard_id)
        if res is not None:
            report_warnings(res)
            report_errors(res)

    @classmethod
    def _web_view(cls, args):
        dash_id = json.load(args.file)["id"]
        url = api._api_host + "/dash/dash/{0}".format(dash_id)
        webbrowser.open(url)

    @classmethod
    def _escape(cls, s):
        return s.replace("\r", "\\r").replace("\n", "\\n").replace("\t", "\\t") if s else ""


def _template_variables(tpl_var_input):
    if "[" not in tpl_var_input:
        return [v.strip() for v in tpl_var_input.split(",")]
    else:
        try:
            return json.loads(tpl_var_input)
        except Exception:
            raise argparse.ArgumentTypeError("bad template_variable json parameter")



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import json

# datadog
from datadog import api
from datadog.dogshell.common import report_errors, report_warnings


# TODO IS there a test ?
class SearchClient(object):
    @classmethod
    def setup_parser(cls, subparsers):
        parser = subparsers.add_parser("search", help="search datadog")
        verb_parsers = parser.add_subparsers(title="Verbs", dest="verb")
        verb_parsers.required = True

        query_parser = verb_parsers.add_parser("query", help="Search datadog.")
        query_parser.add_argument("query", help="optionally faceted search query")
        query_parser.set_defaults(func=cls._query)

    @classmethod
    def _query(cls, args):
        api._timeout = args.timeout
        res = api.Infrastructure.search(q=args.query)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            for facet, results in list(res["results"].items()):
                for idx, result in enumerate(results):
                    if idx == 0:
                        print("\n")
                        print("%s\t%s" % (facet, result))
                    else:
                        print("%s\t%s" % (" " * len(facet), result))
        elif format == "raw":
            print(json.dumps(res))
        else:
            for facet, results in list(res["results"].items()):
                for result in results:
                    print("%s\t%s" % (facet, result))



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.exceptions import ApiError
from datadog.api.resources import (
    CreateableAPIResource,
    GetableAPIResource,
    ActionAPIResource,
    UpdatableAPISyntheticsResource,
    UpdatableAPISyntheticsSubResource,
    ActionAPISyntheticsResource,
)


class Synthetics(
    ActionAPIResource,
    ActionAPISyntheticsResource,
    CreateableAPIResource,
    GetableAPIResource,
    UpdatableAPISyntheticsResource,
    UpdatableAPISyntheticsSubResource,
):
    """
    A wrapper around Sythetics HTTP API.
    """

    _resource_name = "synthetics"
    _sub_resource_name = "status"

    @classmethod
    def get_test(cls, id, **params):
        """
        Get test's details.

        :param id: public id of the test to retrieve
        :type id: string

        :returns: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/tests/<public_test_id>

        name = "tests"

        return super(Synthetics, cls)._trigger_synthetics_class_action("GET", id=id, name=name, params=params)

    @classmethod
    def get_all_tests(cls, **params):
        """
        Get all tests' details.

        :returns: Dictionary representing the API's JSON response
        """

        for p in ["locations", "tags"]:
            if p in params and isinstance(params[p], list):
                params[p] = ",".join(params[p])

        # API path = "synthetics/tests"

        return super(Synthetics, cls).get(id="tests", params=params)

    @classmethod
    def get_devices(cls, **params):
        """
        Get a list of devices for browser checks

        :returns: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/browser/devices"

        name = "browser/devices"

        return super(Synthetics, cls)._trigger_synthetics_class_action("GET", name=name, params=params)

    @classmethod
    def get_locations(cls, **params):
        """
        Get a list of all available locations

        :return: Dictionary representing the API's JSON response
        """

        name = "locations"

        # API path = "synthetics/locations

        return super(Synthetics, cls)._trigger_synthetics_class_action("GET", name=name, params=params)

    @classmethod
    def get_results(cls, id, **params):
        """
        Get the most recent results for a test

        :param id: public id of the test to retrieve results for
        :type id: id

        :return: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/tests/<public_test_id>/results

        path = "tests/{}/results".format(id)

        return super(Synthetics, cls)._trigger_synthetics_class_action("GET", path, params=params)

    @classmethod
    def get_result(cls, id, result_id, **params):
        """
        Get a specific result for a given test.

        :param id: public ID of the test to retrieve the most recent result for
        :type id: id

        :param result_id: result ID of the test to retrieve the most recent result for
        :type result_id: id

        :returns: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/tests/results/<result_id>

        path = "tests/{}/results/{}".format(id, result_id)

        return super(Synthetics, cls)._trigger_synthetics_class_action("GET", path, params=params)

    @classmethod
    def create_test(cls, **params):
        """
        Create a test

        :param name: A unique name for the test
        :type name: string

        :param type: The type of test. Valid values are api and browser
        :type type: string

        :param subtype: required for SSL test - For a SSL API test, specify ssl as the value.
        :Otherwise, you should omit this argument.
        :type subtype: string

        :param config: The test configuration, contains the request specification and the assertions.
        :type config: dict

        :param options: List of options to customize the test
        :type options: dict

        :param message: A description of the test
        :type message: string

        :param locations: A list of the locations to send the tests from
        :type locations: list

        :param tags: A list of tags used to filter the test
        :type tags: list

        :return: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/tests"

        return super(Synthetics, cls).create(id="tests", **params)

    @classmethod
    def edit_test(cls, id, **params):
        """
        Edit a test

        :param id: Public id of the test to edit
        :type id: string

        :return: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/tests/<public_test_id>"

        return super(Synthetics, cls).update_synthetics(id=id, **params)

    @classmethod
    def start_or_pause_test(cls, id, **body):
        """
        Pause a given test

        :param id: public id of the test to pause
        :type id: string

        :param new_status: mew status for the test
        :type id: string

        :returns: Dictionary representing the API's JSON response
        """

        # API path = "synthetics/tests/<public_test_id>/status"

        return super(Synthetics, cls).update_synthetics_items(id=id, **body)

    @classmethod
    def delete_test(cls, **body):
        """
        Delete a test

        :param public_ids: list of public IDs to delete corresponding tests
        :type public_ids: list of strings

        :return: Dictionary representing the API's JSON response
        """

        if not isinstance(body["public_ids"], list):
            raise ApiError("Parameter 'public_ids' must be a list")

        # API path = "synthetics/tests/delete

        return super(Synthetics, cls)._trigger_action("POST", name="synthetics", id="tests/delete", **body)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""
Available HTTP Client for Datadog API client.

Priority:
1. `requests` 3p module
2. `urlfetch` 3p module - Google App Engine only
"""
# stdlib
import copy
import logging
import platform
import urllib
from threading import Lock

# 3p
try:
    import requests
    import requests.adapters
except ImportError:
    requests = None  # type: ignore

try:
    from google.appengine.api import urlfetch, urlfetch_errors
except ImportError:
    urlfetch, urlfetch_errors = None, None

# datadog
from datadog.api.exceptions import ProxyError, ClientError, HTTPError, HttpTimeout


log = logging.getLogger("datadog.api")


def _get_user_agent_header():
    from datadog import version

    return "datadogpy/{version} (python {pyver}; os {os}; arch {arch})".format(
        version=version.__version__,
        pyver=platform.python_version(),
        os=platform.system().lower(),
        arch=platform.machine().lower(),
    )


def _remove_context(exc):
    """Python3: remove context from chained exceptions to prevent leaking API keys in tracebacks."""
    exc.__cause__ = None
    return exc


class HTTPClient(object):
    """
    An abstract generic HTTP client. Subclasses must implement the `request` methods.
    """

    @classmethod
    def request(cls, method, url, headers, params, data, timeout, proxies, verify, max_retries):
        """
        Main method to be implemented by HTTP clients.

        The returned data structure has the following fields:
        * `content`: string containing the response from the server
        * `status_code`: HTTP status code returned by the server

        Can raise the following exceptions:
        * `ClientError`: server cannot be contacted
        * `HttpTimeout`: connection timed out
        * `HTTPError`: unexpected HTTP response code
        """
        raise NotImplementedError(u"Must be implemented by HTTPClient subclasses.")


class RequestClient(HTTPClient):
    """
    HTTP client based on 3rd party `requests` module, using a single session.
    This allows us to keep the session alive to spare some execution time.
    """

    _session = None
    _session_lock = Lock()

    @classmethod
    def request(cls, method, url, headers, params, data, timeout, proxies, verify, max_retries):
        try:

            with cls._session_lock:
                if cls._session is None:
                    cls._session = requests.Session()
                    http_adapter = requests.adapters.HTTPAdapter(max_retries=max_retries)
                    cls._session.mount("https://", http_adapter)
                    cls._session.headers.update({"User-Agent": _get_user_agent_header()})

            result = cls._session.request(
                method, url, headers=headers, params=params, data=data, timeout=timeout, proxies=proxies, verify=verify
            )

            result.raise_for_status()

        except requests.exceptions.ProxyError as e:
            raise _remove_context(ProxyError(method, url, e))
        except requests.ConnectionError as e:
            raise _remove_context(ClientError(method, url, e))
        except requests.exceptions.Timeout:
            raise _remove_context(HttpTimeout(method, url, timeout))
        except requests.exceptions.HTTPError as e:
            if e.response.status_code in (400, 401, 403, 404, 409, 429):
                # This gets caught afterwards and raises an ApiError exception
                pass
            else:
                raise _remove_context(HTTPError(e.response.status_code, result.reason))
        except TypeError:
            raise TypeError(
                u"Your installed version of `requests` library seems not compatible with"
                u"Datadog's usage. We recommand upgrading it ('pip install -U requests')."
                u"If you need help or have any question, please contact support@datadoghq.com"
            )

        return result


class URLFetchClient(HTTPClient):
    """
    HTTP client based on Google App Engine `urlfetch` module.
    """

    @classmethod
    def request(cls, method, url, headers, params, data, timeout, proxies, verify, max_retries):
        """
        Wrapper around `urlfetch.fetch` method.

        TO IMPLEMENT:
        * `max_retries`
        """
        # No local certificate file can be used on Google App Engine
        validate_certificate = True if verify else False

        # Encode parameters in the url
        url_with_params = "{url}?{params}".format(url=url, params=urllib.urlencode(params))
        newheaders = copy.deepcopy(headers)
        newheaders["User-Agent"] = _get_user_agent_header()

        try:
            result = urlfetch.fetch(
                url=url_with_params,
                method=method,
                headers=newheaders,
                validate_certificate=validate_certificate,
                deadline=timeout,
                payload=data,
                # setting follow_redirects=False may be slightly faster:
                # https://cloud.google.com/appengine/docs/python/microservice-performance#use_the_shortest_route
                follow_redirects=False,
            )

            cls.raise_on_status(result)

        except urlfetch.DownloadError as e:
            raise ClientError(method, url, e)
        except urlfetch_errors.DeadlineExceededError:
            raise HttpTimeout(method, url, timeout)

        return result

    @classmethod
    def raise_on_status(cls, result):
        """
        Raise on HTTP status code errors.
        """
        status_code = result.status_code

        if (status_code / 100) != 2:
            if status_code in (400, 401, 403, 404, 409, 429):
                pass
            else:
                raise HTTPError(status_code)


def resolve_http_client():
    """
    Resolve an appropriate HTTP client based the defined priority and user environment.
    """
    if requests:
        log.debug(u"Use `requests` based HTTP client.")
        return RequestClient

    if urlfetch and urlfetch_errors:
        log.debug(u"Use `urlfetch` based HTTP client.")
        return URLFetchClient

    raise ImportError(
        u"Datadog API client was unable to resolve a HTTP client. " u" Please install `requests` library."
    )



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import json
import sys

# 3p
import argparse

# datadog
from datadog import api
from datadog.dogshell.common import report_errors, report_warnings
from datadog.util.format import pretty_json


class DashboardClient(object):
    @classmethod
    def setup_parser(cls, subparsers):
        parser = subparsers.add_parser("dashboard", help="Create, edit, and delete dashboards")

        verb_parsers = parser.add_subparsers(title="Verbs", dest="verb")
        verb_parsers.required = True

        post_parser = verb_parsers.add_parser("post", help="Create dashboards")
        # Required arguments:
        post_parser.add_argument("title", help="title for the new dashboard")
        post_parser.add_argument(
            "widgets", help="widget definitions as a JSON string. If unset," " reads from stdin.", nargs="?"
        )
        post_parser.add_argument("layout_type", choices=["ordered", "free"], help="Layout type of the dashboard.")
        # Optional arguments:
        post_parser.add_argument("--description", help="Short description of the dashboard")
        post_parser.add_argument(
            "--read_only",
            help="Whether this dashboard is read-only. " "If True, only the author and admins can make changes to it.",
            action="store_true",
        )
        post_parser.add_argument(
            "--notify_list",
            type=_json_string,
            help="A json list of user handles, e.g. " '\'["user1@domain.com", "user2@domain.com"]\'',
        )
        post_parser.add_argument(
            "--template_variables",
            type=_json_string,
            help="A json list of template variable dicts, e.g. "
            '\'[{"name": "host", "prefix": "host", '
            '"default": "my-host"}]\'',
        )
        post_parser.set_defaults(func=cls._post)

        update_parser = verb_parsers.add_parser("update", help="Update existing dashboards")
        # Required arguments:
        update_parser.add_argument("dashboard_id", help="Dashboard to replace" " with the new definition")
        update_parser.add_argument("title", help="New title for the dashboard")
        update_parser.add_argument(
            "widgets", help="Widget definitions as a JSON string." " If unset, reads from stdin", nargs="?"
        )
        update_parser.add_argument("layout_type", choices=["ordered", "free"], help="Layout type of the dashboard.")
        # Optional arguments:
        update_parser.add_argument("--description", help="Short description of the dashboard")
        update_parser.add_argument(
            "--read_only",
            help="Whether this dashboard is read-only. " "If True, only the author and admins can make changes to it.",
            action="store_true",
        )
        update_parser.add_argument(
            "--notify_list",
            type=_json_string,
            help="A json list of user handles, e.g. " '\'["user1@domain.com", "user2@domain.com"]\'',
        )
        update_parser.add_argument(
            "--template_variables",
            type=_json_string,
            help="A json list of template variable dicts, e.g. "
            '\'[{"name": "host", "prefix": "host", '
            '"default": "my-host"}]\'',
        )
        update_parser.set_defaults(func=cls._update)

        show_parser = verb_parsers.add_parser("show", help="Show a dashboard definition")
        show_parser.add_argument("dashboard_id", help="Dashboard to show")
        show_parser.set_defaults(func=cls._show)

        delete_parser = verb_parsers.add_parser("delete", help="Delete dashboards")
        delete_parser.add_argument("dashboard_id", help="Dashboard to delete")
        delete_parser.set_defaults(func=cls._delete)

    @classmethod
    def _post(cls, args):
        api._timeout = args.timeout
        format = args.format
        widgets = args.widgets
        if args.widgets is None:
            widgets = sys.stdin.read()
        widgets = json.loads(widgets)

        # Required arguments
        payload = {"title": args.title, "widgets": widgets, "layout_type": args.layout_type}
        # Optional arguments
        if args.description:
            payload["description"] = args.description
        if args.read_only:
            payload["is_read_only"] = args.read_only
        if args.notify_list:
            payload["notify_list"] = args.notify_list
        if args.template_variables:
            payload["template_variables"] = args.template_variables

        res = api.Dashboard.create(**payload)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _update(cls, args):
        api._timeout = args.timeout
        format = args.format
        widgets = args.widgets
        if args.widgets is None:
            widgets = sys.stdin.read()
        widgets = json.loads(widgets)

        # Required arguments
        payload = {"title": args.title, "widgets": widgets, "layout_type": args.layout_type}
        # Optional arguments
        if args.description:
            payload["description"] = args.description
        if args.read_only:
            payload["is_read_only"] = args.read_only
        if args.notify_list:
            payload["notify_list"] = args.notify_list
        if args.template_variables:
            payload["template_variables"] = args.template_variables

        res = api.Dashboard.update(args.dashboard_id, **payload)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _show(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Dashboard.get(args.dashboard_id)
        report_warnings(res)
        report_errors(res)

        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _delete(cls, args):
        api._timeout = args.timeout
        res = api.Dashboard.delete(args.dashboard_id)
        if res is not None:
            report_warnings(res)
            report_errors(res)


def _json_string(str):
    try:
        return json.loads(str)
    except Exception:
        raise argparse.ArgumentTypeError("bad json parameter")



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""

Wraps shell commands and sends the result to Datadog as events. Ex:

dogwrap -n test-job -k $API_KEY --submit_mode all "ls -lah"

Note that you need to enclose your command in quotes to prevent python
from thinking the command line arguments belong to the python command
instead of the wrapped command.

You can also have the script only send events if they fail:

dogwrap -n test-job -k $API_KEY --submit_mode errors "ls -lah"

And you can give the command a timeout too:

dogwrap -n test-job -k $API_KEY --timeout=1 "sleep 3"

"""
# stdlib
from __future__ import print_function

import os
from copy import copy
import optparse
import subprocess
import sys
import threading
import time
import warnings

# datadog
from datadog import initialize, api, __version__
from datadog.util.compat import is_p3k


SUCCESS = "success"
ERROR = "error"
WARNING = "warning"

MAX_EVENT_BODY_LENGTH = 3000


class Timeout(Exception):
    pass


class OutputReader(threading.Thread):
    """
    Thread collecting the output of a subprocess, optionally forwarding it to
    a given file descriptor and storing it for further retrieval.
    """

    def __init__(self, proc_out, fwd_out=None):
        """
        Instantiates an OutputReader.
        :param proc_out: the output to read
        :type proc_out: file descriptor
        :param fwd_out: the output to forward to (None to disable forwarding)
        :type fwd_out: file descriptor or None
        """
        threading.Thread.__init__(self)
        self.daemon = True
        self._out_content = b""
        self._out = proc_out
        self._fwd_out = fwd_out

    def run(self):
        """
        Thread's main loop: collects the output optionnally forwarding it to
        the file descriptor passed in the constructor.
        """
        for line in iter(self._out.readline, b""):
            if self._fwd_out is not None:
                self._fwd_out.write(line)
            self._out_content += line
        self._out.close()

    @property
    def content(self):
        """
        The content stored in out so far. (Not threadsafe, wait with .join())
        """
        return self._out_content


def poll_proc(proc, sleep_interval, timeout):
    """
    Polls the process until it returns or a given timeout has been reached
    """
    start_time = time.time()
    returncode = None
    while returncode is None:
        returncode = proc.poll()
        if time.time() - start_time > timeout:
            raise Timeout()
        else:
            time.sleep(sleep_interval)
    return returncode


def execute(cmd, cmd_timeout, sigterm_timeout, sigkill_timeout, proc_poll_interval, buffer_outs):
    """
    Launches the process and monitors its outputs
    """
    start_time = time.time()
    returncode = -1
    stdout = b""
    stderr = b""
    try:
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    except Exception:
        print(u"Failed to execute %s" % (repr(cmd)), file=sys.stderr)
        raise
    try:
        # Let's that the threads collecting the output from the command in the
        # background
        stdout_buffer = sys.stdout.buffer if is_p3k() else sys.stdout
        stderr_buffer = sys.stderr.buffer if is_p3k() else sys.stderr
        out_reader = OutputReader(proc.stdout, stdout_buffer if not buffer_outs else None)
        err_reader = OutputReader(proc.stderr, stderr_buffer if not buffer_outs else None)
        out_reader.start()
        err_reader.start()

        # Let's quietly wait from the program's completion here to get the exit
        # code when it finishes
        returncode = poll_proc(proc, proc_poll_interval, cmd_timeout)
    except Timeout:
        returncode = Timeout
        sigterm_start = time.time()
        print("Command timed out after %.2fs, killing with SIGTERM" % (time.time() - start_time), file=sys.stderr)
        try:
            proc.terminate()
            try:
                poll_proc(proc, proc_poll_interval, sigterm_timeout)
            except Timeout:
                print(
                    "SIGTERM timeout failed after %.2fs, killing with SIGKILL" % (time.time() - sigterm_start),
                    file=sys.stderr,
                )
                sigkill_start = time.time()
                proc.kill()
                try:
                    poll_proc(proc, proc_poll_interval, sigkill_timeout)
                except Timeout:
                    print(
                        "SIGKILL timeout failed after %.2fs, exiting" % (time.time() - sigkill_start), file=sys.stderr
                    )
        except OSError as e:
            # Ignore OSError 3: no process found.
            if e.errno != 3:
                raise

    # Let's harvest the outputs collected by our background threads
    # after making sure they're done reading it.
    out_reader.join()
    err_reader.join()
    stdout = out_reader.content
    stderr = err_reader.content

    duration = time.time() - start_time

    return returncode, stdout, stderr, duration


def trim_text(text, max_len):
    """
    Trim input text to fit the `max_len` condition.

    If trim is needed: keep the first 1/3rd of the budget on the top,
    and the other 2 thirds on the bottom.
    """
    if len(text) <= max_len:
        return text

    trimmed_text = (
        u"{top_third}\n"
        u"```\n"
        u"*...trimmed...*\n"
        u"```\n"
        u"{bottom_two_third}\n".format(
            top_third=text[: max_len // 3], bottom_two_third=text[len(text) - (2 * max_len) // 3 :]
        )
    )

    return trimmed_text


def build_event_body(cmd, returncode, stdout, stderr, notifications):
    """
    Format and return an event body.

    Note: do not exceed MAX_EVENT_BODY_LENGTH length.
    """
    fmt_stdout = u""
    fmt_stderr = u""
    fmt_notifications = u""

    max_length = MAX_EVENT_BODY_LENGTH // 2 if stdout and stderr else MAX_EVENT_BODY_LENGTH

    if stdout:
        fmt_stdout = u"**>>>> STDOUT <<<<**\n```\n{stdout} \n```\n".format(
            stdout=trim_text(stdout.decode("utf-8", "replace"), max_length)
        )

    if stderr:
        fmt_stderr = u"**>>>> STDERR <<<<**\n```\n{stderr} \n```\n".format(
            stderr=trim_text(stderr.decode("utf-8", "replace"), max_length)
        )

    if notifications:
        notifications = notifications.decode("utf-8", "replace") if isinstance(notifications, bytes) else notifications
        fmt_notifications = u"**>>>> NOTIFICATIONS <<<<**\n\n {notifications}\n".format(notifications=notifications)

    return (
        u"%%%\n"
        u"**>>>> CMD <<<<**\n```\n{command} \n```\n"
        u"**>>>> EXIT CODE <<<<**\n\n {returncode}\n\n\n"
        u"{stdout}"
        u"{stderr}"
        u"{notifications}"
        u"%%%\n".format(
            command=cmd,
            returncode=returncode,
            stdout=fmt_stdout,
            stderr=fmt_stderr,
            notifications=fmt_notifications,
        )
    )


def generate_warning_codes(option, opt, options_warning):
    try:
        # options_warning is a string e.g.: --warning_codes 123,456,789
        # we need to create a list from it
        warning_codes = options_warning.split(",")
        return warning_codes
    except ValueError:
        raise optparse.OptionValueError("option %s: invalid warning codes value(s): %r" % (opt, options_warning))


class DogwrapOption(optparse.Option):
    # https://docs.python.org/3.7/library/optparse.html#adding-new-types
    TYPES = optparse.Option.TYPES + ("warning_codes",)
    TYPE_CHECKER = copy(optparse.Option.TYPE_CHECKER)
    TYPE_CHECKER["warning_codes"] = generate_warning_codes


def parse_options(raw_args=None):
    """
    Parse the raw command line options into an options object and the remaining command string
    """
    parser = optparse.OptionParser(
        usage='%prog -n [event_name] -k [api_key] --submit_mode \
[ all | errors | warnings] [options] "command". \n\nNote that you need to enclose your command in \
quotes to prevent python executing as soon as there is a space in your command. \n \nNOTICE: In \
normal mode, the whole stderr is printed before stdout, in flush_live mode they will be mixed but \
there is not guarantee that messages sent by the command on both stderr and stdout are printed in \
the order they were sent.',
        version="%prog {0}".format(__version__),
        option_class=DogwrapOption,
    )

    parser.add_option(
        "-n",
        "--name",
        action="store",
        type="string",
        help="the name of the event \
as it should appear on your Datadog stream",
    )
    parser.add_option(
        "-k",
        "--api_key",
        action="store",
        type="string",
        help="your DataDog API Key",
        default=os.environ.get("DD_API_KEY"),
    )
    parser.add_option(
        "-s",
        "--site",
        action="store",
        type="choice",
        default="datadoghq.com",
        choices=["datadoghq.com", "us", "datadoghq.eu", "eu"],
        help="The site \
to send data, us (datadoghq.com) or eu (datadoghq.eu), default: us",
    )
    parser.add_option(
        "-m",
        "--submit_mode",
        action="store",
        type="choice",
        default="errors",
        choices=["errors", "warnings", "all"],
        help="[ all | errors | warnings ] if set \
to error, an event will be sent only of the command exits with a non zero exit status or if it \
times out. If set to warning, a list of exit codes need to be provided",
    )
    parser.add_option(
        "--warning_codes",
        action="store",
        type="warning_codes",
        dest="warning_codes",
        help="comma separated list of warning codes, e.g: 127,255",
    )
    parser.add_option(
        "-p",
        "--priority",
        action="store",
        type="choice",
        choices=["normal", "low"],
        help="the priority of the event (default: 'normal')",
    )
    parser.add_option(
        "-t",
        "--timeout",
        action="store",
        type="int",
        default=60 * 60 * 24,
        help="(in seconds)  a timeout after which your command must be aborted. An \
event will be sent to your DataDog stream (default: 24hours)",
    )
    parser.add_option(
        "--sigterm_timeout",
        action="store",
        type="int",
        default=60 * 2,
        help="(in seconds)  When your command times out, the \
process it triggers is sent a SIGTERM. If this sigterm_timeout is reached, it will be sent a \
SIGKILL signal. (default: 2m)",
    )
    parser.add_option(
        "--sigkill_timeout",
        action="store",
        type="int",
        default=60,
        help="(in seconds) how long to wait at most after SIGKILL \
                              has been sent (default: 60s)",
    )
    parser.add_option(
        "--proc_poll_interval",
        action="store",
        type="float",
        default=0.5,
        help="(in seconds). interval at which your command will be polled \
(default: 500ms)",
    )
    parser.add_option(
        "--notify_success",
        action="store",
        type="string",
        default="",
        help="a message string and @people directives to send notifications in \
case of success.",
    )
    parser.add_option(
        "--notify_error",
        action="store",
        type="string",
        default="",
        help="a message string and @people directives to send notifications in \
case of error.",
    )
    parser.add_option(
        "--notify_warning",
        action="store",
        type="string",
        default="",
        help="a message string and @people directives to send notifications in \
    case of warning.",
    )
    parser.add_option(
        "-b",
        "--buffer_outs",
        action="store_true",
        dest="buffer_outs",
        default=False,
        help="displays the stderr and stdout of the command only once it has \
returned (the command outputs remains buffered in dogwrap meanwhile)",
    )
    parser.add_option(
        "--send_metric",
        action="store_true",
        dest="send_metric",
        default=False,
        help="sends a metric for event duration",
    )
    parser.add_option(
        "--tags", action="store", type="string", dest="tags", default="", help="comma separated list of tags"
    )

    options, args = parser.parse_args(args=raw_args)

    if is_p3k():
        cmd = " ".join(args)
    else:
        cmd = b" ".join(args).decode("utf-8")

    return options, cmd


def main():
    options, cmd = parse_options()

    # If silent is checked we force the outputs to be buffered (and therefore
    # not forwarded to the Terminal streams) and we just avoid printing the
    # buffers at the end
    returncode, stdout, stderr, duration = execute(
        cmd,
        options.timeout,
        options.sigterm_timeout,
        options.sigkill_timeout,
        options.proc_poll_interval,
        options.buffer_outs,
    )

    if options.site in ("datadoghq.eu", "eu"):
        api_host = "https://api.datadoghq.eu"
    else:
        api_host = "https://api.datadoghq.com"

    initialize(api_key=options.api_key, api_host=api_host)
    host = api._host_name

    warning_codes = None

    if options.warning_codes:
        # Convert warning codes from string to int since return codes will evaluate the latter
        warning_codes = list(map(int, options.warning_codes))

    if returncode == 0:
        alert_type = SUCCESS
        event_priority = "low"
        event_title = u"[%s] %s succeeded in %.2fs" % (host, options.name, duration)
    elif returncode != 0 and options.submit_mode == "warnings":
        if not warning_codes:
            # the list of warning codes is empty - the option was not specified
            print("A comma separated list of exit codes need to be provided")
            sys.exit()
        elif returncode in warning_codes:
            alert_type = WARNING
            event_priority = "normal"
            event_title = u"[%s] %s failed in %.2fs" % (host, options.name, duration)
        else:
            print("Command exited with a different exit code that the one(s) provided")
            sys.exit()
    else:
        alert_type = ERROR
        event_priority = "normal"

        if returncode is Timeout:
            event_title = u"[%s] %s timed out after %.2fs" % (host, options.name, duration)
            returncode = -1
        else:
            event_title = u"[%s] %s failed in %.2fs" % (host, options.name, duration)

    notifications = ""

    if alert_type == SUCCESS and options.notify_success:
        notifications = options.notify_success
    elif alert_type == ERROR and options.notify_error:
        notifications = options.notify_error
    elif alert_type == WARNING and options.notify_warning:
        notifications = options.notify_warning

    if options.tags:
        tags = [t.strip() for t in options.tags.split(",")]
    else:
        tags = None

    event_body = build_event_body(cmd, returncode, stdout, stderr, notifications)

    event = {
        "alert_type": alert_type,
        "aggregation_key": options.name,
        "host": host,
        "priority": options.priority or event_priority,
        "tags": tags,
    }

    if options.buffer_outs:
        if is_p3k():
            stderr = stderr.decode("utf-8")
            stdout = stdout.decode("utf-8")

        print(stderr.strip(), file=sys.stderr)
        print(stdout.strip(), file=sys.stdout)

    if options.submit_mode == "all" or returncode != 0:
        if options.send_metric:
            event_name_tag = "event_name:{}".format(options.name)
            if tags:
                duration_tags = tags + [event_name_tag]
            else:
                duration_tags = [event_name_tag]
            api.Metric.send(metric="dogwrap.duration", points=duration, tags=duration_tags, type="gauge")
        api.Event.create(title=event_title, text=event_body, **event)

    sys.exit(returncode)


if __name__ == "__main__":
    if sys.argv[0].endswith("dogwrap"):
        warnings.warn("dogwrap is pending deprecation. Please use dogshellwrap instead.", PendingDeprecationWarning)
    main()



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""
Reporter classes.
"""


from datadog import api


class Reporter(object):
    def flush(self, metrics):
        raise NotImplementedError()


class HttpReporter(Reporter):
    def __init__(self, compress_payload=False):
        self.compress_payload = compress_payload

    def flush_distributions(self, distributions):
        api.Distribution.send(distributions, compress_payload=self.compress_payload)

    def flush_metrics(self, metrics):
        api.Metric.send(metrics, compress_payload=self.compress_payload)

    def flush_events(self, events):
        for event in events:
            api.Event.create(**event)


class GraphiteReporter(Reporter):
    def flush(self, metrics):
        pass



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""
Datadogpy is a collection of Datadog Python tools.
It contains:
* datadog.api: a Python client for Datadog REST API.
* datadog.dogstatsd: a DogStatsd Python client.
* datadog.threadstats: an alternative tool to DogStatsd client for collecting application metrics
without hindering performance.
* datadog.dogshell: a command-line tool, wrapping datadog.api, to interact with Datadog REST API.
"""
# stdlib
import logging
import os
import os.path
from typing import Any, List, Optional

# datadog
from datadog import api
from datadog.dogstatsd import DogStatsd, statsd  # noqa
from datadog.threadstats import ThreadStats, datadog_lambda_wrapper, lambda_metric  # noqa
from datadog.util.compat import iteritems, NullHandler, text
from datadog.util.hostname import get_hostname
from datadog.version import __version__  # noqa

# Loggers
logging.getLogger("datadog.api").addHandler(NullHandler())
logging.getLogger("datadog.dogstatsd").addHandler(NullHandler())
logging.getLogger("datadog.threadstats").addHandler(NullHandler())


def initialize(
    api_key=None,  # type: Optional[str]
    app_key=None,  # type: Optional[str]
    host_name=None,  # type: Optional[str]
    api_host=None,  # type: Optional[str]
    statsd_host=None,  # type: Optional[str]
    statsd_port=None,  # type: Optional[int]
    statsd_use_default_route=False,  # type: bool
    statsd_socket_path=None,  # type: Optional[str]
    statsd_namespace=None,  # type: Optional[str]
    statsd_constant_tags=None,  # type: Optional[List[str]]
    return_raw_response=False,  # type: bool
    hostname_from_config=True,  # type: bool
    **kwargs  # type: Any
):
    # type: (...) -> None
    """
    Initialize and configure Datadog.api and Datadog.statsd modules

    :param api_key: Datadog API key
    :type api_key: string

    :param app_key: Datadog application key
    :type app_key: string

    :param host_name: Set a specific hostname
    :type host_name: string

    :param proxies: Proxy to use to connect to Datadog API;
                    for example, 'proxies': {'http': "http:<user>:<pass>@<ip>:<port>/"}
    :type proxies: dictionary mapping protocol to the URL of the proxy.

    :param api_host: Datadog API endpoint
    :type api_host: url

    :param statsd_host: Host of DogStatsd server or statsd daemon
    :type statsd_host: address

    :param statsd_port: Port of DogStatsd server or statsd daemon
    :type statsd_port: port

    :param statsd_use_default_route: Dynamically set the statsd host to the default route
                                     (Useful when running the client in a container)
    :type statsd_use_default_route: boolean

    :param statsd_socket_path: path to the DogStatsd UNIX socket. Supersedes statsd_host
                               and stats_port if provided.

    :param statsd_constant_tags: A list of tags to be applied to all metrics ("tag", "tag:value")
    :type statsd_constant_tags: list of string

    :param cacert: Path to local certificate file used to verify SSL \
        certificates. Can also be set to True (default) to use the systems \
        certificate store, or False to skip SSL verification
    :type cacert: path or boolean

    :param mute: Mute any ApiError or ClientError before they escape \
        from datadog.api.HTTPClient (default: True).
    :type mute: boolean

    :param return_raw_response: Whether or not to return the raw response object in addition \
        to the decoded response content (default: False)
    :type return_raw_response: boolean

    :param hostname_from_config: Set the hostname from the Datadog agent config (agent 5). Will be deprecated
    :type hostname_from_config: boolean
    """
    # API configuration
    api._api_key = api_key or api._api_key or os.environ.get("DATADOG_API_KEY", os.environ.get("DD_API_KEY"))
    api._application_key = (
        app_key or api._application_key or os.environ.get("DATADOG_APP_KEY", os.environ.get("DD_APP_KEY"))
    )
    api._hostname_from_config = hostname_from_config
    api._host_name = host_name or api._host_name or get_hostname(hostname_from_config)
    api._api_host = api_host or api._api_host or os.environ.get("DATADOG_HOST", "https://api.datadoghq.com")

    # Statsd configuration
    # ...overrides the default `statsd` instance attributes
    if statsd_socket_path:
        statsd.socket_path = statsd_socket_path
        statsd.host = None
        statsd.port = None
    else:
        if statsd_host or statsd_use_default_route:
            statsd.host = statsd.resolve_host(statsd_host, statsd_use_default_route)
        if statsd_port:
            statsd.port = int(statsd_port)
    if statsd_namespace:
        statsd.namespace = text(statsd_namespace)
    if statsd_constant_tags:
        statsd.constant_tags += statsd_constant_tags

    api._return_raw_response = return_raw_response

    # HTTP client and API options
    for key, value in iteritems(kwargs):
        attribute = "_{}".format(key)
        setattr(api, attribute, value)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import argparse
import json

# 3p
from datadog.util.format import pretty_json

# datadog
from datadog import api
from datadog.dogshell.common import report_errors, report_warnings, print_err


class MonitorClient(object):
    @classmethod
    def setup_parser(cls, subparsers):
        parser = subparsers.add_parser("monitor", help="Create, edit, and delete monitors")
        parser.add_argument(
            "--string_ids",
            action="store_true",
            dest="string_ids",
            help="Represent monitor IDs as strings instead of ints in JSON",
        )

        verb_parsers = parser.add_subparsers(title="Verbs", dest="verb")
        verb_parsers.required = True

        post_parser = verb_parsers.add_parser("post", help="Create a monitor")
        post_parser.add_argument("type", help="type of the monitor, e.g." "'metric alert' 'service check'")
        post_parser.add_argument(
            "query", help="query to notify on with syntax varying " "depending on what type of monitor you are creating"
        )
        post_parser.add_argument("--name", help="name of the alert", default=None)
        post_parser.add_argument(
            "--message", help="message to include with notifications" " for this monitor", default=None
        )
        post_parser.add_argument("--tags", help="comma-separated list of tags", default=None)
        post_parser.add_argument("--options", help="json options for the monitor", default=None)
        post_parser.set_defaults(func=cls._post)

        file_post_parser = verb_parsers.add_parser("fpost", help="Create a monitor from file")
        file_post_parser.add_argument("file", help="json file holding all details", type=argparse.FileType("r"))
        file_post_parser.set_defaults(func=cls._file_post)

        update_parser = verb_parsers.add_parser("update", help="Update existing monitor")
        update_parser.add_argument("monitor_id", help="monitor to replace with the new definition")
        update_parser.add_argument(
            "type",
            nargs="?",
            help="[Deprecated] optional argument preferred" "type of the monitor, e.g. 'metric alert' 'service check'",
            default=None,
        )
        update_parser.add_argument(
            "query",
            nargs="?",
            help="[Deprecated] optional argument preferred"
            "query to notify on with syntax varying depending on monitor type",
            default=None,
        )
        update_parser.add_argument(
            "--type", help="type of the monitor, e.g. " "'metric alert' 'service check'", default=None, dest="type_opt"
        )
        update_parser.add_argument(
            "--query",
            help="query to notify on with syntax varying" " depending on monitor type",
            default=None,
            dest="query_opt",
        )
        update_parser.add_argument("--name", help="name of the alert", default=None)
        update_parser.add_argument(
            "--message", help="message to include with " "notifications for this monitor", default=None
        )
        update_parser.add_argument("--options", help="json options for the monitor", default=None)
        update_parser.set_defaults(func=cls._update)

        file_update_parser = verb_parsers.add_parser("fupdate", help="Update existing" " monitor from file")
        file_update_parser.add_argument("file", help="json file holding all details", type=argparse.FileType("r"))
        file_update_parser.set_defaults(func=cls._file_update)

        show_parser = verb_parsers.add_parser("show", help="Show a monitor definition")
        show_parser.add_argument("monitor_id", help="monitor to show")
        show_parser.set_defaults(func=cls._show)

        show_all_parser = verb_parsers.add_parser("show_all", help="Show a list of all monitors")
        show_all_parser.add_argument(
            "--group_states",
            help="comma separated list of group states to filter by"
            "(choose one or more from 'all', 'alert', 'warn', or 'no data')",
        )
        show_all_parser.add_argument("--name", help="string to filter monitors by name")
        show_all_parser.add_argument(
            "--tags",
            help="comma separated list indicating what tags, if any, "
            "should be used to filter the list of monitors by scope (e.g. 'host:host0')",
        )
        show_all_parser.add_argument(
            "--monitor_tags",
            help="comma separated list indicating what service "
            "and/or custom tags, if any, should be used to filter the list of monitors",
        )

        show_all_parser.set_defaults(func=cls._show_all)

        delete_parser = verb_parsers.add_parser("delete", help="Delete a monitor")
        delete_parser.add_argument("monitor_id", help="monitor to delete")
        delete_parser.set_defaults(func=cls._delete)

        mute_all_parser = verb_parsers.add_parser("mute_all", help="Globally mute " "monitors (downtime over *)")
        mute_all_parser.set_defaults(func=cls._mute_all)

        unmute_all_parser = verb_parsers.add_parser(
            "unmute_all", help="Globally unmute " "monitors (cancel downtime over *)"
        )
        unmute_all_parser.set_defaults(func=cls._unmute_all)

        mute_parser = verb_parsers.add_parser("mute", help="Mute a monitor")
        mute_parser.add_argument("monitor_id", help="monitor to mute")
        mute_parser.add_argument("--scope", help="scope to apply the mute to," " e.g. role:db (optional)", default=[])
        mute_parser.add_argument(
            "--end", help="POSIX timestamp for when" " the mute should end (optional)", default=None
        )
        mute_parser.set_defaults(func=cls._mute)

        unmute_parser = verb_parsers.add_parser("unmute", help="Unmute a monitor")
        unmute_parser.add_argument("monitor_id", help="monitor to unmute")
        unmute_parser.add_argument("--scope", help="scope to unmute (must be muted), " "e.g. role:db", default=[])
        unmute_parser.add_argument("--all_scopes", help="clear muting across all scopes", action="store_true")
        unmute_parser.set_defaults(func=cls._unmute)

        can_delete_parser = verb_parsers.add_parser("can_delete", help="Check if you can delete some monitors")
        can_delete_parser.add_argument("monitor_ids", help="monitors to check if they can be deleted")
        can_delete_parser.set_defaults(func=cls._can_delete)

        validate_parser = verb_parsers.add_parser("validate", help="Validates if a monitor definition is correct")
        validate_parser.add_argument("type", help="type of the monitor, e.g." "'metric alert' 'service check'")
        validate_parser.add_argument("query", help="the monitor query")
        validate_parser.add_argument("--name", help="name of the alert", default=None)
        validate_parser.add_argument(
            "--message", help="message to include with notifications" " for this monitor", default=None
        )
        validate_parser.add_argument("--tags", help="comma-separated list of tags", default=None)
        validate_parser.add_argument("--options", help="json options for the monitor", default=None)
        validate_parser.set_defaults(func=cls._validate)

    @classmethod
    def _post(cls, args):
        api._timeout = args.timeout
        format = args.format
        options = None
        if args.options is not None:
            options = json.loads(args.options)

        if args.tags:
            tags = sorted(set([t.strip() for t in args.tags.split(",") if t.strip()]))
        else:
            tags = None

        res = api.Monitor.create(
            type=args.type, query=args.query, name=args.name, message=args.message, tags=tags, options=options
        )
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _file_post(cls, args):
        api._timeout = args.timeout
        format = args.format
        monitor = json.load(args.file)
        res = api.Monitor.create(
            type=monitor["type"],
            query=monitor["query"],
            name=monitor["name"],
            message=monitor["message"],
            options=monitor["options"],
        )
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _update(cls, args):
        api._timeout = args.timeout
        format = args.format

        to_update = {}
        if args.type:
            if args.type_opt:
                msg = "Duplicate arguments for `type`. Using optional value --type"
                print_err("WARNING: {}".format(msg))
            else:
                to_update["type"] = args.type
            msg = "[DEPRECATION] `type` is no longer required to `update` and may be omitted"
            print_err("WARNING: {}".format(msg))
        if args.query:
            if args.query_opt:
                msg = "Duplicate arguments for `query`. Using optional value --query"
                print_err("WARNING: {}".format(msg))
            else:
                to_update["query"] = args.query
            msg = "[DEPRECATION] `query` is no longer required to `update` and may be omitted"
            print_err("WARNING: {}".format(msg))
        if args.name:
            to_update["name"] = args.name
        if args.message:
            to_update["message"] = args.message
        if args.type_opt:
            to_update["type"] = args.type_opt
        if args.query_opt:
            to_update["query"] = args.query_opt

        if args.options is not None:
            to_update["options"] = json.loads(args.options)

        res = api.Monitor.update(args.monitor_id, **to_update)

        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _file_update(cls, args):
        api._timeout = args.timeout
        format = args.format
        monitor = json.load(args.file)
        res = api.Monitor.update(
            monitor["id"],
            type=monitor["type"],
            query=monitor["query"],
            name=monitor["name"],
            message=monitor["message"],
            options=monitor["options"],
        )
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _show(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Monitor.get(args.monitor_id)
        report_warnings(res)
        report_errors(res)

        if args.string_ids:
            res["id"] = str(res["id"])

        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _show_all(cls, args):
        api._timeout = args.timeout
        format = args.format

        res = api.Monitor.get_all(
            group_states=args.group_states, name=args.name, tags=args.tags, monitor_tags=args.monitor_tags
        )
        report_warnings(res)
        report_errors(res)

        if args.string_ids:
            for d in res:
                d["id"] = str(d["id"])

        if format == "pretty":
            print(pretty_json(res))
        elif format == "raw":
            print(json.dumps(res))
        else:
            for d in res:
                print(
                    "\t".join(
                        [
                            (str(d["id"])),
                            (cls._escape(d["message"])),
                            (cls._escape(d["name"])),
                            (str(d["options"])),
                            (str(d["org_id"])),
                            (d["query"]),
                            (d["type"]),
                        ]
                    )
                )

    @classmethod
    def _delete(cls, args):
        api._timeout = args.timeout
        # TODO CHECK
        res = api.Monitor.delete(args.monitor_id)
        if res is not None:
            report_warnings(res)
            report_errors(res)

    @classmethod
    def _escape(cls, s):
        return s.replace("\r", "\\r").replace("\n", "\\n").replace("\t", "\\t")

    @classmethod
    def _mute_all(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Monitor.mute_all()
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _unmute_all(cls, args):
        api._timeout = args.timeout
        res = api.Monitor.unmute_all()
        if res is not None:
            report_warnings(res)
            report_errors(res)

    @classmethod
    def _mute(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Monitor.mute(args.monitor_id, scope=args.scope, end=args.end)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _unmute(cls, args):
        api._timeout = args.timeout
        res = api.Monitor.unmute(args.monitor_id, scope=args.scope, all_scopes=args.all_scopes)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _can_delete(cls, args):
        api._timeout = args.timeout
        monitor_ids = [i.strip() for i in args.monitor_ids.split(",") if i.strip()]
        res = api.Monitor.can_delete(monitor_ids=monitor_ids)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _validate(cls, args):
        api._timeout = args.timeout
        format = args.format
        options = None
        if args.options is not None:
            options = json.loads(args.options)

        if args.tags:
            tags = sorted(set([t.strip() for t in args.tags.split(",") if t.strip()]))
        else:
            tags = None

        res = api.Monitor.validate(
            type=args.type, query=args.query, name=args.name, message=args.message, tags=tags, options=options
        )
        # report_warnings(res)
        # report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.exceptions import ApiError
from datadog.api.resources import GetableAPIResource, CreateableAPIResource, SearchableAPIResource
from datadog.util.compat import iteritems


class Event(GetableAPIResource, CreateableAPIResource, SearchableAPIResource):
    """
    A wrapper around Event HTTP API.
    """

    _resource_name = "events"
    _timestamp_keys = set(["start", "end"])

    @classmethod
    def create(cls, attach_host_name=True, **params):
        """
        Post an event.

        :param title: title for the new event
        :type title: string

        :param text: event message
        :type text: string

        :param aggregation_key: key by which to group events in event stream
        :type aggregation_key: string

        :param alert_type: "error", "warning", "info" or "success".
        :type alert_type: string

        :param date_happened: when the event occurred. if unset defaults to the current time. \
        (POSIX timestamp)
        :type date_happened: integer

        :param handle: user to post the event as. defaults to owner of the application key used \
        to submit.
        :type handle: string

        :param priority: priority to post the event as. ("normal" or "low", defaults to "normal")
        :type priority: string

        :param related_event_id: post event as a child of the given event
        :type related_event_id: id

        :param tags: tags to post the event with
        :type tags: list of strings

        :param host: host to post the event with
        :type host: string

        :param device_name: device_name to post the event with
        :type device_name: list of strings

        :returns: Dictionary representing the API's JSON response

        >>> title = "Something big happened!"
        >>> text = 'And let me tell you all about it here!'
        >>> tags = ['version:1', 'application:web']

        >>> api.Event.create(title=title, text=text, tags=tags)
        """
        if params.get("alert_type"):
            if params["alert_type"] not in ["error", "warning", "info", "success"]:
                raise ApiError("Parameter alert_type must be either error, warning, info or success")

        return super(Event, cls).create(attach_host_name=attach_host_name, **params)

    @classmethod
    def query(cls, **params):
        """
        Get the events that occurred between the *start* and *end* POSIX timestamps,
        optional filtered by *priority* ("low" or "normal"), *sources* and
        *tags*.

        See the `event API documentation <http://docs.datadoghq.com/api/#events-get-all>`_ for the
        event data format.

        :returns: Dictionary representing the API's JSON response

        >>> api.Event.query(start=1313769783, end=1419436870, priority="normal", \
            tags=["application:web"])
        """

        def timestamp_to_integer(k, v):
            if k in cls._timestamp_keys:
                return int(v)
            else:
                return v

        params = dict((k, timestamp_to_integer(k, v)) for k, v in iteritems(params))

        return super(Event, cls)._search(**params)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# datadog
from datadog.api.format import format_points
from datadog.api.resources import SendableAPIResource


class Distribution(SendableAPIResource):
    """A wrapper around Distribution HTTP API"""

    _resource_name = "distribution_points"

    @classmethod
    def send(cls, distributions=None, attach_host_name=True, compress_payload=False, **distribution):
        """
        Submit a distribution metric or a list of distribution metrics to the distribution metric
        API

        :param compress_payload: compress the payload using zlib
        :type compress_payload: bool
        :param metric: the name of the time series
        :type metric: string
        :param points: a (timestamp, [list of values]) pair or
        list of (timestamp, [list of values]) pairs
        :type points: list
        :param host: host name that produced the metric
        :type host: string
        :param tags:  list of tags associated with the metric.
        :type tags: string list
        :returns: Dictionary representing the API's JSON response
        """
        if distributions:
            # Multiple distributions are sent
            for d in distributions:
                if isinstance(d, dict):
                    d["points"] = format_points(d["points"])
            series_dict = {"series": distributions}
        else:
            # One distribution is sent
            distribution["points"] = format_points(distribution["points"])
            series_dict = {"series": [distribution]}
        return super(Distribution, cls).send(
            attach_host_name=attach_host_name, compress_payload=compress_payload, **series_dict
        )



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import (
    GetableAPIResource,
    CreateableAPIResource,
    UpdatableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    ActionAPIResource,
)


class Downtime(
    GetableAPIResource,
    CreateableAPIResource,
    UpdatableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    ActionAPIResource,
):
    """
    A wrapper around Monitor Downtiming HTTP API.
    """

    _resource_name = "downtime"

    @classmethod
    def cancel_downtime_by_scope(cls, **body):
        """
        Cancels all downtimes matching the scope.

        :param scope: scope to cancel downtimes by
        :type scope: string

        :returns: Dictionary representing the API's JSON response
        """
        return super(Downtime, cls)._trigger_class_action("POST", "cancel/by_scope", **body)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import CreateableAPIResource
from datadog.api.api_client import APIClient


class Logs(CreateableAPIResource):
    """
    A wrapper around Log HTTP API.
    """

    _resource_name = "logs-queries"

    @classmethod
    def list(cls, data):
        path = "{resource_name}/list".format(
            resource_name=cls._resource_name,
        )
        api_version = getattr(cls, "_api_version", None)

        return APIClient.submit("POST", path, api_version, data)



#!/usr/bin/env python

# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""
DogStatsd is a Python client for DogStatsd, a Statsd fork for Datadog.
"""
# Standard libraries
from random import random
import logging
import os
import socket
import errno
import time
from threading import Lock, RLock

# Datadog libraries
from datadog.dogstatsd.context import (
    TimedContextManagerDecorator,
    DistributedContextManagerDecorator,
)
from datadog.dogstatsd.route import get_default_route
from datadog.util.compat import is_p3k, text
from datadog.util.format import normalize_tags
from datadog.version import __version__
from typing import Optional, List, Text, Union

# Logging
log = logging.getLogger("datadog.dogstatsd")

# Default config
DEFAULT_HOST = "localhost"
DEFAULT_PORT = 8125

# Tag name of entity_id
ENTITY_ID_TAG_NAME = "dd.internal.entity_id"

# Default buffer settings
UDP_OPTIMAL_PAYLOAD_LENGTH = 1432
UDS_OPTIMAL_PAYLOAD_LENGTH = 8192

# Mapping of each "DD_" prefixed environment variable to a specific tag name
DD_ENV_TAGS_MAPPING = {
    "DD_ENTITY_ID": ENTITY_ID_TAG_NAME,
    "DD_ENV": "env",
    "DD_SERVICE": "service",
    "DD_VERSION": "version",
}

# Telemetry minimum flush interval in seconds
DEFAULT_TELEMETRY_MIN_FLUSH_INTERVAL = 10

# Telemetry pre-computed formatting string. Pre-computation
# increases throughput of composing the result by 2-15% from basic
# '%'-based formatting with a `join`.
TELEMETRY_FORMATTING_STR = "\n".join(
    [
        "datadog.dogstatsd.client.metrics:%s|c|#%s",
        "datadog.dogstatsd.client.events:%s|c|#%s",
        "datadog.dogstatsd.client.service_checks:%s|c|#%s",
        "datadog.dogstatsd.client.bytes_sent:%s|c|#%s",
        "datadog.dogstatsd.client.bytes_dropped:%s|c|#%s",
        "datadog.dogstatsd.client.packets_sent:%s|c|#%s",
        "datadog.dogstatsd.client.packets_dropped:%s|c|#%s",
    ]
)


class DogStatsd(object):
    OK, WARNING, CRITICAL, UNKNOWN = (0, 1, 2, 3)

    def __init__(
        self,
        host=DEFAULT_HOST,  # type: Text
        port=DEFAULT_PORT,  # type: int
        max_buffer_size=None,  # type: None
        namespace=None,  # type: Optional[Text]
        constant_tags=None,  # type: Optional[List[str]]
        use_ms=False,  # type: bool
        use_default_route=False,  # type: bool
        socket_path=None,  # type: Optional[Text]
        default_sample_rate=1,  # type: float
        disable_telemetry=False,  # type: bool
        telemetry_min_flush_interval=(DEFAULT_TELEMETRY_MIN_FLUSH_INTERVAL),  # type: int
        telemetry_host=None,  # type: Text
        telemetry_port=None,  # type: Union[str, int]
        telemetry_socket_path=None,  # type: Text
        max_buffer_len=0,  # type: int
    ):  # type: (...) -> None
        """
        Initialize a DogStatsd object.

        >>> statsd = DogStatsd()

        :envvar DD_AGENT_HOST: the host of the DogStatsd server.
        If set, it overrides default value.
        :type DD_AGENT_HOST: string

        :envvar DD_DOGSTATSD_PORT: the port of the DogStatsd server.
        If set, it overrides default value.
        :type DD_DOGSTATSD_PORT: integer

        :envvar DATADOG_TAGS: Tags to attach to every metric reported by dogstatsd client.
        :type DATADOG_TAGS: comma-delimited string

        :envvar DD_ENTITY_ID: Tag to identify the client entity.
        :type DD_ENTITY_ID: string

        :envvar DD_ENV: the env of the service running the dogstatsd client.
        If set, it is appended to the constant (global) tags of the statsd client.
        :type DD_ENV: string

        :envvar DD_SERVICE: the name of the service running the dogstatsd client.
        If set, it is appended to the constant (global) tags of the statsd client.
        :type DD_SERVICE: string

        :envvar DD_VERSION: the version of the service running the dogstatsd client.
        If set, it is appended to the constant (global) tags of the statsd client.
        :type DD_VERSION: string

        :envvar DD_DOGSTATSD_DISABLE: Disable any statsd metric collection (default False)
        :type DD_DOGSTATSD_DISABLE: boolean

        :envvar DD_TELEMETRY_HOST: the host for the dogstatsd server we wish to submit
        telemetry stats to. If set, it overrides default value.
        :type DD_TELEMETRY_HOST: string

        :envvar DD_TELEMETRY_PORT: the port for the dogstatsd server we wish to submit
        telemetry stats to. If set, it overrides default value.
        :type DD_TELEMETRY_PORT: integer

        :param host: the host of the DogStatsd server.
        :type host: string

        :param port: the port of the DogStatsd server.
        :type port: integer

        :max_buffer_size: Deprecated option, do not use it anymore.
        :type max_buffer_type: None

        :param namespace: Namespace to prefix all metric names
        :type namespace: string

        :param constant_tags: Tags to attach to all metrics
        :type constant_tags: list of strings

        :param use_ms: Report timed values in milliseconds instead of seconds (default False)
        :type use_ms: boolean

        :param use_default_route: Dynamically set the DogStatsd host to the default route
        (Useful when running the client in a container) (Linux only)
        :type use_default_route: boolean

        :param socket_path: Communicate with dogstatsd through a UNIX socket instead of
        UDP. If set, disables UDP transmission (Linux only)
        :type socket_path: string

        :param default_sample_rate: Sample rate to use by default for all metrics
        :type default_sample_rate: float

        :param max_buffer_len: Maximum number of bytes to buffer before sending to the server
        if sending metrics in batch. If not specified it will be adjusted to a optimal value
        depending on the connection type.
        :type max_buffer_len: integer

        :param disable_telemetry: Should client telemetry be disabled
        :type disable_telemetry: boolean

        :param telemetry_min_flush_interval: Minimum flush interval for telemetry in seconds
        :type telemetry_min_flush_interval: integer

        :param telemetry_host: the host for the dogstatsd server we wish to submit
        telemetry stats to. Optional. If telemetry is enabled and this is not specified
        the default host will be used.
        :type host: string

        :param telemetry_port: the port for the dogstatsd server we wish to submit
        telemetry stats to. Optional. If telemetry is enabled and this is not specified
        the default host will be used.
        :type port: integer

        :param telemetry_socket_path: Submit client telemetry to dogstatsd through a UNIX
        socket instead of UDP. If set, disables UDP transmission (Linux only)
        :type telemetry_socket_path: string
        """

        self._socket_lock = Lock()

        # Check for deprecated option
        if max_buffer_size is not None:
            log.warning("The parameter max_buffer_size is now deprecated and is not used anymore")

        # Check host and port env vars
        agent_host = os.environ.get("DD_AGENT_HOST")
        if agent_host and host == DEFAULT_HOST:
            host = agent_host

        dogstatsd_port = os.environ.get("DD_DOGSTATSD_PORT")
        if dogstatsd_port and port == DEFAULT_PORT:
            try:
                port = int(dogstatsd_port)
            except ValueError:
                log.warning(
                    "Port number provided in DD_DOGSTATSD_PORT env var is not an integer: \
                %s, using %s as port number",
                    dogstatsd_port,
                    port,
                )

        # Assuming environment variables always override
        telemetry_host = os.environ.get("DD_TELEMETRY_HOST", telemetry_host)
        telemetry_port = os.environ.get("DD_TELEMETRY_PORT", telemetry_port) or port

        # Check enabled
        if os.environ.get("DD_DOGSTATSD_DISABLE") not in {"True", "true", "yes", "1"}:
            self._enabled = True
        else:
            self._enabled = False

        # Connection
        self._max_payload_size = max_buffer_len
        if socket_path is not None:
            self.socket_path = socket_path  # type: Optional[text]
            self.host = None
            self.port = None
            transport = "uds"
            if not self._max_payload_size:
                self._max_payload_size = UDS_OPTIMAL_PAYLOAD_LENGTH
        else:
            self.socket_path = None
            self.host = self.resolve_host(host, use_default_route)
            self.port = int(port)
            transport = "udp"
            if not self._max_payload_size:
                self._max_payload_size = UDP_OPTIMAL_PAYLOAD_LENGTH

        self.telemetry_socket_path = telemetry_socket_path
        self.telemetry_host = None
        self.telemetry_port = None
        if not telemetry_socket_path and telemetry_host:
            self.telemetry_socket_path = None
            self.telemetry_host = self.resolve_host(telemetry_host, use_default_route)
            self.telemetry_port = int(telemetry_port)

        # Socket
        self.socket = None
        self.telemetry_socket = None
        self._send = self._send_to_server
        self.encoding = "utf-8"

        # Options
        env_tags = [tag for tag in os.environ.get("DATADOG_TAGS", "").split(",") if tag]
        # Inject values of DD_* environment variables as global tags.
        for var, tag_name in DD_ENV_TAGS_MAPPING.items():
            value = os.environ.get(var, "")
            if value:
                env_tags.append("{name}:{value}".format(name=tag_name, value=value))
        if constant_tags is None:
            constant_tags = []
        self.constant_tags = constant_tags + env_tags
        if namespace is not None:
            namespace = text(namespace)
        self.namespace = namespace
        self.use_ms = use_ms
        self.default_sample_rate = default_sample_rate

        # init telemetry version
        self._client_tags = [
            "client:py",
            "client_version:{}".format(__version__),
            "client_transport:{}".format(transport),
        ]
        self._reset_telemetry()
        self._telemetry_flush_interval = telemetry_min_flush_interval
        self._telemetry = not disable_telemetry

        self._buffer_lock = RLock()

    def disable_telemetry(self):
        self._telemetry = False

    def enable_telemetry(self):
        self._telemetry = True

    def _dedicated_telemetry_destination(self):
        return bool(self.telemetry_socket_path or self.telemetry_host)

    def __enter__(self):
        self.open_buffer()
        return self

    def __exit__(self, type, value, traceback):
        self.close_buffer()

    @staticmethod
    def resolve_host(host, use_default_route):
        """
        Resolve the DogStatsd host.

        :param host: host
        :type host: string
        :param use_default_route: use the system default route as host (overrides the `host` parameter)
        :type use_default_route: bool
        """
        if not use_default_route:
            return host

        return get_default_route()

    def get_socket(self, telemetry=False):
        """
        Return a connected socket.

        Note: connect the socket before assigning it to the class instance to
        avoid bad thread race conditions.
        """
        with self._socket_lock:
            if telemetry and self._dedicated_telemetry_destination():
                if not self.telemetry_socket:
                    if self.telemetry_socket_path is not None:
                        self.telemetry_socket = self._get_uds_socket(self.telemetry_socket_path)
                    else:
                        self.telemetry_socket = self._get_udp_socket_socket(self.telemetry_host, self.telemetry_port)

                return self.telemetry_socket

            if not self.socket:
                if self.socket_path is not None:
                    self.socket = self._get_uds_socket(self.socket_path)
                else:
                    self.socket = self._get_udp_socket(self.host, self.port)

            return self.socket

    @staticmethod
    def _get_uds_socket(socket_path):
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        sock.setblocking(0)
        sock.connect(socket_path)
        return sock

    @staticmethod
    def _get_udp_socket(host, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setblocking(0)
        sock.connect((host, port))
        return sock

    def open_buffer(self, max_buffer_size=None):
        """
        Open a buffer to send a batch of metrics.

        You can also use this as a context manager.

        >>> with DogStatsd() as batch:
        >>>     batch.gauge("users.online", 123)
        >>>     batch.gauge("active.connections", 1001)

        Note: This method must be called before close_buffer() matching invocation.
        """

        self._buffer_lock.acquire()

        if max_buffer_size is not None:
            log.warning("The parameter max_buffer_size is now deprecated and is not used anymore")
        self._current_buffer_total_size = 0
        self.buffer = []
        self._send = self._send_to_buffer

    def close_buffer(self):
        """
        Flush the buffer and switch back to single metric packets.

        Note: This method must be called after a matching open_buffer()
        invocation.
        """

        if not hasattr(self, "buffer"):
            raise BufferError("Cannot close buffer that was never opened")

        try:
            self._send = self._send_to_server

            if self.buffer:
                # Only send packets if there are packets to send
                self._flush_buffer()
        finally:
            self._buffer_lock.release()

    def gauge(
        self,
        metric,  # type: Text
        value,  # type: float
        tags=None,  # type: Optional[List[str]]
        sample_rate=None,  # type: Optional[float]
    ):  # type(...) -> None
        """
        Record the value of a gauge, optionally setting a list of tags and a
        sample rate.

        >>> statsd.gauge("users.online", 123)
        >>> statsd.gauge("active.connections", 1001, tags=["protocol:http"])
        """
        return self._report(metric, "g", value, tags, sample_rate)

    def increment(
        self,
        metric,  # type: Text
        value=1,  # type: float
        tags=None,  # type: Optional[List[str]]
        sample_rate=None,  # type: Optional[float]
    ):  # type: (...) -> None
        """
        Increment a counter, optionally setting a value, tags and a sample
        rate.

        >>> statsd.increment("page.views")
        >>> statsd.increment("files.transferred", 124)
        """
        self._report(metric, "c", value, tags, sample_rate)

    def decrement(self, metric, value=1, tags=None, sample_rate=None):
        """
        Decrement a counter, optionally setting a value, tags and a sample
        rate.

        >>> statsd.decrement("files.remaining")
        >>> statsd.decrement("active.connections", 2)
        """
        metric_value = -value if value else value
        self._report(metric, "c", metric_value, tags, sample_rate)

    def histogram(self, metric, value, tags=None, sample_rate=None):
        """
        Sample a histogram value, optionally setting tags and a sample rate.

        >>> statsd.histogram("uploaded.file.size", 1445)
        >>> statsd.histogram("album.photo.count", 26, tags=["gender:female"])
        """
        self._report(metric, "h", value, tags, sample_rate)

    def distribution(self, metric, value, tags=None, sample_rate=None):
        """
        Send a global distribution value, optionally setting tags and a sample rate.

        >>> statsd.distribution("uploaded.file.size", 1445)
        >>> statsd.distribution("album.photo.count", 26, tags=["gender:female"])
        """
        self._report(metric, "d", value, tags, sample_rate)

    def timing(self, metric, value, tags=None, sample_rate=None):
        """
        Record a timing, optionally setting tags and a sample rate.

        >>> statsd.timing("query.response.time", 1234)
        """
        self._report(metric, "ms", value, tags, sample_rate)

    def timed(self, metric=None, tags=None, sample_rate=None, use_ms=None):
        """
        A decorator or context manager that will measure the distribution of a
        function's/context's run time. Optionally specify a list of tags or a
        sample rate. If the metric is not defined as a decorator, the module
        name and function name will be used. The metric is required as a context
        manager.
        ::

            @statsd.timed("user.query.time", sample_rate=0.5)
            def get_user(user_id):
                # Do what you need to ...
                pass

            # Is equivalent to ...
            with statsd.timed("user.query.time", sample_rate=0.5):
                # Do what you need to ...
                pass

            # Is equivalent to ...
            start = time.time()
            try:
                get_user(user_id)
            finally:
                statsd.timing("user.query.time", time.time() - start)
        """
        return TimedContextManagerDecorator(self, metric, tags, sample_rate, use_ms)

    def distributed(self, metric=None, tags=None, sample_rate=None, use_ms=None):
        """
        A decorator or context manager that will measure the distribution of a
        function's/context's run time using custom metric distribution.
        Optionally specify a list of tags or a sample rate. If the metric is not
        defined as a decorator, the module name and function name will be used.
        The metric is required as a context manager.
        ::

            @statsd.distributed("user.query.time", sample_rate=0.5)
            def get_user(user_id):
                # Do what you need to ...
                pass

            # Is equivalent to ...
            with statsd.distributed("user.query.time", sample_rate=0.5):
                # Do what you need to ...
                pass

            # Is equivalent to ...
            start = time.time()
            try:
                get_user(user_id)
            finally:
                statsd.distribution("user.query.time", time.time() - start)
        """
        return DistributedContextManagerDecorator(self, metric, tags, sample_rate, use_ms)

    def set(self, metric, value, tags=None, sample_rate=None):
        """
        Sample a set value.

        >>> statsd.set("visitors.uniques", 999)
        """
        self._report(metric, "s", value, tags, sample_rate)

    def close_socket(self):
        """
        Closes connected socket if connected.
        """
        with self._socket_lock:
            if self.socket:
                try:
                    self.socket.close()
                except OSError as e:
                    log.error("Unexpected error: %s", str(e))
                self.socket = None

            if self.telemetry_socket:
                try:
                    self.telemetry_socket.close()
                except OSError as e:
                    log.error("Unexpected error: %s", str(e))
                self.telemetry_socket = None

    def _serialize_metric(self, metric, metric_type, value, tags, sample_rate=1):
        # Create/format the metric packet
        return "%s%s:%s|%s%s%s" % (
            (self.namespace + ".") if self.namespace else "",
            metric,
            value,
            metric_type,
            ("|@" + text(sample_rate)) if sample_rate != 1 else "",
            ("|#" + ",".join(normalize_tags(tags))) if tags else "",
        )

    def _report(self, metric, metric_type, value, tags, sample_rate):
        """
        Create a metric packet and send it.

        More information about the packets' format: http://docs.datadoghq.com/guides/dogstatsd/
        """
        if value is None:
            return

        if self._enabled is not True:
            return

        if self._telemetry:
            self.metrics_count += 1

        if sample_rate is None:
            sample_rate = self.default_sample_rate

        if sample_rate != 1 and random() > sample_rate:
            return

        # Resolve the full tag list
        tags = self._add_constant_tags(tags)
        payload = self._serialize_metric(metric, metric_type, value, tags, sample_rate)

        # Send it
        self._send(payload)

    def _reset_telemetry(self):
        self.metrics_count = 0
        self.events_count = 0
        self.service_checks_count = 0
        self.bytes_sent = 0
        self.bytes_dropped = 0
        self.packets_sent = 0
        self.packets_dropped = 0
        self._last_flush_time = time.time()

    def _flush_telemetry(self):
        telemetry_tags = ",".join(self._add_constant_tags(self._client_tags))

        return TELEMETRY_FORMATTING_STR % (
            self.metrics_count,
            telemetry_tags,
            self.events_count,
            telemetry_tags,
            self.service_checks_count,
            telemetry_tags,
            self.bytes_sent,
            telemetry_tags,
            self.bytes_dropped,
            telemetry_tags,
            self.packets_sent,
            telemetry_tags,
            self.packets_dropped,
            telemetry_tags,
        )

    def _is_telemetry_flush_time(self):
        return self._telemetry and self._last_flush_time + self._telemetry_flush_interval < time.time()

    def _send_to_server(self, packet):
        self._xmit_packet(packet, False)
        if self._is_telemetry_flush_time():
            telemetry = self._flush_telemetry()
            if self._xmit_packet(telemetry, True):
                self._reset_telemetry()
                self.packets_sent += 1
                self.bytes_sent += len(telemetry)
            else:
                # Telemetry packet has been dropped, keep telemetry data for the next flush
                self._last_flush_time = time.time()
                self.bytes_dropped += len(telemetry)
                self.packets_dropped += 1

    def _xmit_packet(self, packet, is_telemetry):
        try:
            if is_telemetry and self._dedicated_telemetry_destination():
                mysocket = self.telemetry_socket or self.get_socket(telemetry=True)
            else:
                # If set, use socket directly
                mysocket = self.socket or self.get_socket()

            mysocket.send(packet.encode(self.encoding))

            if not is_telemetry and self._telemetry:
                self.packets_sent += 1
                self.bytes_sent += len(packet)

            return True
        except socket.timeout:
            # dogstatsd is overflowing, drop the packets (mimicks the UDP behaviour)
            pass
        except (socket.herror, socket.gaierror) as se:
            log.warning(
                "Error submitting packet: %s, dropping the packet and closing the socket",
                se,
            )
            self.close_socket()
        except socket.error as se:
            if se.errno == errno.EAGAIN:
                log.debug("Socket send would block: %s, dropping the packet", se)
            else:
                log.warning(
                    "Error submitting packet: %s, dropping the packet and closing the socket",
                    se,
                )
                self.close_socket()
        except Exception as e:
            log.error("Unexpected error: %s", str(e))
        if not is_telemetry and self._telemetry:
            self.bytes_dropped += len(packet)
            self.packets_dropped += 1
        return False

    def _send_to_buffer(self, packet):
        if self._should_flush(len(packet)):
            self._flush_buffer()
        self.buffer.append(packet)
        # Update the current buffer length, including line break to anticipate the final packet size
        self._current_buffer_total_size += len(packet) + 1

    def _should_flush(self, length_to_be_added):
        if self._current_buffer_total_size + length_to_be_added > self._max_payload_size:
            return True
        return False

    def _flush_buffer(self):
        self._send_to_server("\n".join(self.buffer))
        self._current_buffer_total_size = 0
        self.buffer = []

    def _escape_event_content(self, string):
        return string.replace("\n", "\\n")

    def _escape_service_check_message(self, string):
        return string.replace("\n", "\\n").replace("m:", "m\\:")

    def event(
        self,
        title,
        text,
        alert_type=None,
        aggregation_key=None,
        source_type_name=None,
        date_happened=None,
        priority=None,
        tags=None,
        hostname=None,
    ):
        """
        Send an event. Attributes are the same as the Event API.
            http://docs.datadoghq.com/api/

        >>> statsd.event("Man down!", "This server needs assistance.")
        >>> statsd.event("The web server restarted", "The web server is up again", alert_type="success")  # NOQA
        """
        title = self._escape_event_content(title)
        text = self._escape_event_content(text)

        if not is_p3k():
            if not isinstance(title, unicode):                              # noqa: F821
                title = unicode(self._escape_event_content(title), 'utf8')  # noqa: F821
            if not isinstance(text, unicode):                               # noqa: F821
                text = unicode(self._escape_event_content(text), 'utf8')    # noqa: F821

        # Append all client level tags to every event
        tags = self._add_constant_tags(tags)

        string = u"_e{{{},{}}}:{}|{}".format(
            len(title.encode('utf8', 'replace')),
            len(text.encode('utf8', 'replace')),
            title,
            text,
        )

        if date_happened:
            string = "%s|d:%d" % (string, date_happened)
        if hostname:
            string = "%s|h:%s" % (string, hostname)
        if aggregation_key:
            string = "%s|k:%s" % (string, aggregation_key)
        if priority:
            string = "%s|p:%s" % (string, priority)
        if source_type_name:
            string = "%s|s:%s" % (string, source_type_name)
        if alert_type:
            string = "%s|t:%s" % (string, alert_type)
        if tags:
            string = "%s|#%s" % (string, ",".join(tags))

        if len(string) > 8 * 1024:
            raise Exception(u'Event "%s" payload is too big (more than 8KB), ' "event discarded" % title)

        if self._telemetry:
            self.events_count += 1
        self._send(string)

    def service_check(self, check_name, status, tags=None, timestamp=None, hostname=None, message=None):
        """
        Send a service check run.

        >>> statsd.service_check("my_service.check_name", DogStatsd.WARNING)
        """
        message = self._escape_service_check_message(message) if message is not None else ""

        string = u"_sc|{0}|{1}".format(check_name, status)

        # Append all client level tags to every status check
        tags = self._add_constant_tags(tags)

        if timestamp:
            string = u"{0}|d:{1}".format(string, timestamp)
        if hostname:
            string = u"{0}|h:{1}".format(string, hostname)
        if tags:
            string = u"{0}|#{1}".format(string, ",".join(tags))
        if message:
            string = u"{0}|m:{1}".format(string, message)

        if self._telemetry:
            self.service_checks_count += 1
        self._send(string)

    def _add_constant_tags(self, tags):
        if self.constant_tags:
            if tags:
                return tags + self.constant_tags
            else:
                return self.constant_tags
        return tags


statsd = DogStatsd()



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc


class CheckStatus(object):
    OK = 0
    WARNING = 1
    CRITICAL = 2
    UNKNOWN = 3
    ALL = (OK, WARNING, CRITICAL, UNKNOWN)


class MonitorType(object):
    # From https://docs.datadoghq.com/api/?lang=bash#create-a-monitor
    QUERY_ALERT = "query alert"
    COMPOSITE = "composite"
    SERVICE_CHECK = "service check"
    PROCESS_ALERT = "process alert"
    LOG_ALERT = "log alert"
    METRIC_ALERT = "metric alert"
    RUM_ALERT = "rum alert"
    EVENT_ALERT = "event alert"
    SYNTHETICS_ALERT = "synthetics alert"
    TRACE_ANALYTICS = "trace-analytics alert"



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import json

# 3p
from datadog.util.format import pretty_json

# datadog
from datadog import api
from datadog.dogshell.common import report_errors, report_warnings


class DowntimeClient(object):
    @classmethod
    def setup_parser(cls, subparsers):
        parser = subparsers.add_parser("downtime", help="Create, edit, and delete downtimes")
        parser.add_argument(
            "--string_ids",
            action="store_true",
            dest="string_ids",
            help="Represent downtime IDs as strings instead of ints in JSON",
        )

        verb_parsers = parser.add_subparsers(title="Verbs", dest="verb")
        verb_parsers.required = True

        post_parser = verb_parsers.add_parser("post", help="Create a downtime")
        post_parser.add_argument("scope", help="scope to apply downtime to")
        post_parser.add_argument("start", help="POSIX timestamp to start the downtime", default=None)
        post_parser.add_argument("--end", help="POSIX timestamp to end the downtime", default=None)
        post_parser.add_argument(
            "--message", help="message to include with notifications" " for this downtime", default=None
        )
        post_parser.set_defaults(func=cls._schedule_downtime)

        update_parser = verb_parsers.add_parser("update", help="Update existing downtime")
        update_parser.add_argument("downtime_id", help="downtime to replace" " with the new definition")
        update_parser.add_argument("--scope", help="scope to apply downtime to")
        update_parser.add_argument("--start", help="POSIX timestamp to start" " the downtime", default=None)
        update_parser.add_argument("--end", help="POSIX timestamp to" " end the downtime", default=None)
        update_parser.add_argument(
            "--message", help="message to include with notifications" " for this downtime", default=None
        )
        update_parser.set_defaults(func=cls._update_downtime)

        show_parser = verb_parsers.add_parser("show", help="Show a downtime definition")
        show_parser.add_argument("downtime_id", help="downtime to show")
        show_parser.set_defaults(func=cls._show_downtime)

        show_all_parser = verb_parsers.add_parser("show_all", help="Show a list of all downtimes")
        show_all_parser.add_argument(
            "--current_only", help="only return downtimes that" " are active when the request is made", default=None
        )
        show_all_parser.set_defaults(func=cls._show_all_downtime)

        delete_parser = verb_parsers.add_parser("delete", help="Delete a downtime")
        delete_parser.add_argument("downtime_id", help="downtime to delete")
        delete_parser.set_defaults(func=cls._cancel_downtime)

        cancel_parser = verb_parsers.add_parser("cancel_by_scope", help="Cancel all downtimes with a given scope")
        cancel_parser.add_argument("scope", help="The scope of the downtimes to cancel")
        cancel_parser.set_defaults(func=cls._cancel_downtime_by_scope)

    @classmethod
    def _schedule_downtime(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Downtime.create(scope=args.scope, start=args.start, end=args.end, message=args.message)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _update_downtime(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Downtime.update(
            args.downtime_id, scope=args.scope, start=args.start, end=args.end, message=args.message
        )
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _cancel_downtime(cls, args):
        api._timeout = args.timeout
        res = api.Downtime.delete(args.downtime_id)
        if res is not None:
            report_warnings(res)
            report_errors(res)

    @classmethod
    def _show_downtime(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Downtime.get(args.downtime_id)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _show_all_downtime(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Downtime.get_all(current_only=args.current_only)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))

    @classmethod
    def _cancel_downtime_by_scope(cls, args):
        api._timeout = args.timeout
        format = args.format
        res = api.Downtime.cancel_downtime_by_scope(scope=args.scope)
        if format == "pretty":
            print(pretty_json(res))
        else:
            print(json.dumps(res))



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.threadstats import ThreadStats
from threading import Lock, Thread
from datadog import api
import os
import warnings

"""
DEPRECATED use datadog-lambda package instead https://git.io/fjy8o
Usage:

from datadog import datadog_lambda_wrapper, lambda_metric

@datadog_lambda_wrapper
def my_lambda_handle(event, context):
    lambda_metric("some_metric", 10)
"""


class _LambdaDecorator(object):
    """ DEPRECATED Decorator to automatically init & flush metrics, created for Lambda functions"""

    # Number of opened wrappers, flush when 0
    _counter = 0
    _counter_lock = Lock()
    _flush_lock = Lock()
    _was_initialized = False

    def __init__(self, func):
        self.func = func

    @classmethod
    def _enter(cls):

        with cls._counter_lock:
            if not cls._was_initialized:
                cls._was_initialized = True
                api._api_key = os.environ.get("DATADOG_API_KEY", os.environ.get("DD_API_KEY"))
                api._api_host = os.environ.get("DATADOG_HOST", "https://api.datadoghq.com")

                # Async initialization of the TLS connection with our endpoints
                # This avoids adding execution time at the end of the lambda run
                t = Thread(target=_init_api_client)
                t.start()

                # Make sure the global ThreadStats has been created
                _get_lambda_stats()
            cls._counter = cls._counter + 1

    @classmethod
    def _close(cls):
        should_flush = False
        with cls._counter_lock:
            cls._counter = cls._counter - 1

            # Flush only when all wrappers are closed
            if cls._counter <= 0:
                should_flush = True

        if should_flush:
            with cls._flush_lock:
                # Don't flush if other wrappers were opened while _flush_lock was locked
                with cls._counter_lock:
                    if cls._counter > 0:
                        should_flush = False
                if should_flush:
                    _get_lambda_stats().flush(float("inf"))

    def __call__(self, *args, **kw):
        warnings.warn("datadog_lambda_wrapper() is relocated to https://git.io/fjy8o", DeprecationWarning)
        _LambdaDecorator._enter()
        try:
            return self.func(*args, **kw)
        finally:
            _LambdaDecorator._close()


_lambda_stats = None
datadog_lambda_wrapper = _LambdaDecorator


def _get_lambda_stats():
    global _lambda_stats
    # This is not thread-safe, it should be called first by _LambdaDecorator
    if _lambda_stats is None:
        _lambda_stats = ThreadStats()
        _lambda_stats.start(flush_in_greenlet=False, flush_in_thread=False)
    return _lambda_stats


def lambda_metric(*args, **kw):
    """ Alias to expose only distributions for lambda functions"""
    _get_lambda_stats().distribution(*args, **kw)


def _init_api_client():
    """No-op GET to initialize the requests connection with DD's endpoints

    The goal here is to make the final flush faster:
    we keep alive the Requests session, this means that we can re-use the connection
    The consequence is that the HTTP Handshake, which can take hundreds of ms,
    is now made at the beginning of a lambda instead of at the end.

    By making the initial request async, we spare a lot of execution time in the lambdas.
    """
    try:
        api.api_client.APIClient.submit("GET", "validate")
    except Exception:
        pass



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""
API & HTTP Clients exceptions.
"""


class DatadogException(Exception):
    """
    Base class for Datadog API exceptions.  Use this for patterns like the following:

        try:
            # do something with the Datadog API
        except datadog.api.exceptions.DatadogException:
            # handle any Datadog-specific exceptions
    """


class ProxyError(DatadogException):
    """
    HTTP connection to the configured proxy server failed.
    """

    def __init__(self, method, url, exception):
        message = (
            u"Could not request {method} {url}: Unable to connect to proxy. "
            u"Please check the proxy configuration and try again.".format(method=method, url=url)
        )
        super(ProxyError, self).__init__(message)


class ClientError(DatadogException):
    """
    HTTP connection to Datadog endpoint is not possible.
    """

    def __init__(self, method, url, exception):
        message = (
            u"Could not request {method} {url}: {exception}. "
            u"Please check the network connection or try again later. "
            u"If the problem persists, please contact support@datadoghq.com".format(
                method=method, url=url, exception=exception
            )
        )
        super(ClientError, self).__init__(message)


class HttpTimeout(DatadogException):
    """
    HTTP connection timeout.
    """

    def __init__(self, method, url, timeout):
        message = (
            u"{method} {url} timed out after {timeout}. "
            u"Please try again later. "
            u"If the problem persists, please contact support@datadoghq.com".format(
                method=method, url=url, timeout=timeout
            )
        )
        super(HttpTimeout, self).__init__(message)


class HttpBackoff(DatadogException):
    """
    Backing off after too many timeouts.
    """

    def __init__(self, backoff_period):
        message = u"Too many timeouts. Won't try again for {backoff_period} seconds. ".format(
            backoff_period=backoff_period
        )
        super(HttpBackoff, self).__init__(message)


class HTTPError(DatadogException):
    """
    Datadog returned a HTTP error.
    """

    def __init__(self, status_code=None, reason=None):
        reason = u" - {reason}".format(reason=reason) if reason else u""
        message = (
            u"Datadog returned a bad HTTP response code: {status_code}{reason}. "
            u"Please try again later. "
            u"If the problem persists, please contact support@datadoghq.com".format(
                status_code=status_code,
                reason=reason,
            )
        )

        super(HTTPError, self).__init__(message)


class ApiError(DatadogException):
    """
    Datadog returned an API error (known HTTPError).

    Matches the following status codes: 400, 401, 403, 404, 409, 429.
    """


class ApiNotInitialized(DatadogException):
    "No API key is set"



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import (
    AddableAPISubResource,
    DeletableAPISubResource,
    ListableAPISubResource,
    UpdatableAPISubResource,
)


class DashboardListV2(ListableAPISubResource, AddableAPISubResource, UpdatableAPISubResource, DeletableAPISubResource):
    """
    A wrapper around Dashboard List HTTP API.
    """

    _resource_name = "dashboard/lists/manual"
    _sub_resource_name = "dashboards"
    _api_version = "v2"



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# flake8: noqa

from typing import Optional

# API settings
_api_key = None  # type: Optional[str]
_application_key = None  # type: Optional[str]
_api_version = "v1"
_api_host = None  # type: Optional[str]
_host_name = None  # type: Optional[str]
_hostname_from_config = True
_cacert = True

# HTTP(S) settings
_proxies = None
_timeout = 60
_max_timeouts = 3
_max_retries = 3
_backoff_period = 300
_mute = True
_return_raw_response = False

# Resources
from datadog.api.comments import Comment
from datadog.api.dashboard_lists import DashboardList
from datadog.api.distributions import Distribution
from datadog.api.downtimes import Downtime
from datadog.api.timeboards import Timeboard
from datadog.api.dashboards import Dashboard
from datadog.api.events import Event
from datadog.api.infrastructure import Infrastructure
from datadog.api.metadata import Metadata
from datadog.api.metrics import Metric
from datadog.api.monitors import Monitor
from datadog.api.screenboards import Screenboard
from datadog.api.graphs import Graph, Embed
from datadog.api.hosts import Host, Hosts
from datadog.api.service_checks import ServiceCheck
from datadog.api.tags import Tag
from datadog.api.users import User
from datadog.api.aws_integration import AwsIntegration
from datadog.api.aws_log_integration import AwsLogsIntegration
from datadog.api.azure_integration import AzureIntegration
from datadog.api.gcp_integration import GcpIntegration
from datadog.api.roles import Roles
from datadog.api.permissions import Permissions
from datadog.api.service_level_objectives import ServiceLevelObjective
from datadog.api.synthetics import Synthetics
from datadog.api.logs import Logs



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# flake8: noqa
"""
Imports for compatibility with Python 2, Python 3 and Google App Engine.
"""
from functools import wraps
import logging
import socket
import sys

# Note: using `sys.version_info` instead of the helper functions defined here
# so that mypy detects version-specific code paths. Currently, mypy doesn't
# support try/except imports for version-specific code paths either.
#
# https://mypy.readthedocs.io/en/stable/common_issues.html#python-version-and-system-platform-checks

# Python 3.x
if sys.version_info[0] >= 3:
    import builtins
    from collections import UserDict as IterableUserDict
    import configparser
    from configparser import ConfigParser
    from io import StringIO
    from urllib.parse import urljoin, urlparse
    import urllib.request as url_lib, urllib.error, urllib.parse

    imap = map
    get_input = input
    text = str

    def iteritems(d):
        return iter(d.items())

    def iternext(iter):
        return next(iter)


# Python 2.x
else:
    import __builtin__ as builtins
    import ConfigParser as configparser
    from configparser import ConfigParser
    from cStringIO import StringIO
    from itertools import imap
    import urllib2 as url_lib
    from urlparse import urljoin, urlparse
    from UserDict import IterableUserDict

    get_input = raw_input
    text = unicode

    def iteritems(d):
        return d.iteritems()

    def iternext(iter):
        return iter.next()


# Python >= 3.5
if sys.version_info >= (3, 5):
    from asyncio import iscoroutinefunction
# Others
else:

    def iscoroutinefunction(*args, **kwargs):
        return False


# Python >= 2.7
if sys.version_info >= (2, 7):
    from logging import NullHandler
# Python 2.6.x
else:
    from logging import Handler

    class NullHandler(Handler):
        def emit(self, record):
            pass


def _is_py_version_higher_than(major, minor=0):
    """
    Assert that the Python version is higher than `$maj.$min`.
    """
    return sys.version_info >= (major, minor)


def is_p3k():
    """
    Assert that Python is version 3 or higher.
    """
    return _is_py_version_higher_than(3)


def is_higher_py35():
    """
    Assert that Python is version 3.5 or higher.
    """
    return _is_py_version_higher_than(3, 5)


def is_pypy():
    """
    Assert that PyPy is being used (regardless of 2 or 3)
    """
    return "__pypy__" in sys.builtin_module_names



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
"""
Event aggregator class.
"""

from datadog.util.compat import iteritems


class EventsAggregator(object):
    """
    A simple event aggregator
    """

    def __init__(self):
        self._events = []

    def add_event(self, **event):
        # Clean empty values
        event = dict((k, v) for k, v in iteritems(event) if v is not None)
        self._events.append(event)

    def flush(self):
        events = self._events
        self._events = []
        return events



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import (
    GetableAPIResource,
    CreateableAPIResource,
    DeletableAPIResource,
    UpdatableAPIResource,
    AddableAPISubResource,
)


class AzureIntegration(
    GetableAPIResource, CreateableAPIResource, DeletableAPIResource, UpdatableAPIResource, AddableAPISubResource
):
    """
    A wrapper around Azure integration API.
    """

    _resource_name = "integration"
    _resource_id = "azure"

    @classmethod
    def list(cls, **params):
        """
        List all Datadog-Azure integrations available in your Datadog organization.

        >>> api.AzureIntegration.list()
        """
        return super(AzureIntegration, cls).get(id=cls._resource_id, **params)

    @classmethod
    def create(cls, **params):
        """
        Add a new Azure integration config.

        >>> tenant_name = "<AZURE_TENANT_NAME>"
        >>> client_id = "<AZURE_CLIENT_ID>"
        >>> client_secret = "<AZURE_CLIENT_SECRET>"
        >>> host_filters = ["<KEY>:<VALUE>"]

        >>> api.AzureIntegration.create(tenant_name=tenant_name, client_id=client_id, \
        client_secret=client_secret,host_filters=host_filters)
        """
        return super(AzureIntegration, cls).create(id=cls._resource_id, **params)

    @classmethod
    def delete(cls, **body):
        """
        Delete a given Datadog-Azure integration.

        >>> tenant_name = "<AZURE_TENANT_NAME>"
        >>> client_id = "<AZURE_CLIENT_ID>"

        >>> api.AzureIntegration.delete(tenant_name=tenant_name, client_id=client_id)
        """
        return super(AzureIntegration, cls).delete(id=cls._resource_id, body=body)

    @classmethod
    def update_host_filters(cls, **params):
        """
        Update the defined list of host filters for a given Datadog-Azure integration. \

        >>> tenant_name = "<AZURE_TENANT_NAME>"
        >>> client_id = "<AZURE_CLIENT_ID>"
        >>> host_filters = "<KEY>:<VALUE>"

        >>> api.AzureIntegration.update_host_filters(tenant_name=tenant_name, client_id=client_id, \
            host_filters=host_filters)
        """
        cls._sub_resource_name = "host_filters"
        return super(AzureIntegration, cls).add_items(id=cls._resource_id, **params)

    @classmethod
    def update(cls, **body):
        """
        Update an Azure account configuration.

        >>> tenant_name = "<AZURE_TENANT_NAME>"
        >>> client_id = "<AZURE_CLIENT_ID>"
        >>> new_tenant_name = "<NEW_AZURE_TENANT_NAME>"
        >>> new_client_id = "<NEW_AZURE_CLIENT_ID>"
        >>> client_secret = "<AZURE_CLIENT_SECRET>"
        >>> host_filters = "<KEY>:<VALUE>"

        >>> api.AzureIntegration.update(tenant_name=tenant_name, client_id=client_id, \
        new_tenant_name=new_tenant_name, new_client_id=new_client_id,\
        client_secret=client_secret, host_filters=host_filters)
        """
        params = {}
        return super(AzureIntegration, cls).update(id=cls._resource_id, params=params, **body)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from numbers import Number
import sys
import time

if sys.version_info[0] >= 3:
    from collections.abc import Iterable
else:
    from collections import Iterable


def format_points(points):
    """
    Format `points` parameter.

    Input:
        a value or (timestamp, value) pair or a list of value or (timestamp, value) pairs

    Returns:
        list of (timestamp, float value) pairs

    """
    now = time.time()
    if not isinstance(points, list):
        points = [points]

    formatted_points = []
    for point in points:
        if isinstance(point, Number):
            timestamp = now
            value = float(point)
        # Distributions contain a list of points
        else:
            timestamp = point[0]
            if isinstance(point[1], Iterable):
                value = [float(p) for p in point[1]]
            else:
                value = float(point[1])

        formatted_points.append((timestamp, value))

    return formatted_points



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import SearchableAPIResource


class Infrastructure(SearchableAPIResource):
    """
    A wrapper around Infrastructure HTTP API.
    """

    _resource_name = "search"

    @classmethod
    def search(cls, **params):
        """
        Search for entities in Datadog.

        :param q: a query to search for host and metrics
        :type q: string query

        :returns: Dictionary representing the API's JSON response
        """
        # Deprecate the hosts search param
        query = params.get("q", "").split(":")
        if len(query) > 1 and query[0] == "hosts":
            print("[DEPRECATION] Infrastructure.search() is deprecated for ", "hosts. Use `Hosts.search` instead.")
        return super(Infrastructure, cls)._search(**params)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
# stdlib
import json
import sys

# datadog
from datadog import api
from datadog.dogshell.common import report_errors, report_warnings


class CommentClient(object):
    @classmethod
    def setup_parser(cls, subparsers):
        parser = subparsers.add_parser("comment", help="Post, update, and delete comments.")

        verb_parsers = parser.add_subparsers(title="Verbs", dest="verb")
        verb_parsers.required = True

        post_parser = verb_parsers.add_parser("post", help="Post comments.")
        post_parser.add_argument("handle", help="handle to post as.")
        post_parser.add_argument("comment", help="comment message to post. if unset," " reads from stdin.", nargs="?")
        post_parser.set_defaults(func=cls._post)

        update_parser = verb_parsers.add_parser("update", help="Update existing comments.")
        update_parser.add_argument("comment_id", help="comment to update (by id)")
        update_parser.add_argument("handle", help="handle to post as.")
        update_parser.add_argument("comment", help="comment message to post." " if unset, reads from stdin.", nargs="?")
        update_parser.set_defaults(func=cls._update)

        reply_parser = verb_parsers.add_parser("reply", help="Reply to existing comments.")
        reply_parser.add_argument("comment_id", help="comment to reply to (by id)")
        reply_parser.add_argument("handle", help="handle to post as.")
        reply_parser.add_argument("comment", help="comment message to post." " if unset, reads from stdin.", nargs="?")
        reply_parser.set_defaults(func=cls._reply)

        show_parser = verb_parsers.add_parser("show", help="Show comment details.")
        show_parser.add_argument("comment_id", help="comment to show")
        show_parser.set_defaults(func=cls._show)

    @classmethod
    def _post(cls, args):
        api._timeout = args.timeout
        handle = args.handle
        comment = args.comment
        format = args.format
        if comment is None:
            comment = sys.stdin.read()
        res = api.Comment.create(handle=handle, message=comment)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            message = res["comment"]["message"]
            lines = message.split("\n")
            message = "\n".join(["    " + line for line in lines])
            print("id\t\t" + str(res["comment"]["id"]))
            print("url\t\t" + res["comment"]["url"])
            print("resource\t" + res["comment"]["resource"])
            print("handle\t\t" + res["comment"]["handle"])
            print("message\n" + message)
        elif format == "raw":
            print(json.dumps(res))
        else:
            print("id\t\t" + str(res["comment"]["id"]))
            print("url\t\t" + res["comment"]["url"])
            print("resource\t" + res["comment"]["resource"])
            print("handle\t\t" + res["comment"]["handle"])
            print("message\t\t" + res["comment"]["message"].__repr__())

    @classmethod
    def _update(cls, args):
        handle = args.handle
        comment = args.comment
        id = args.comment_id
        format = args.format
        if comment is None:
            comment = sys.stdin.read()
        res = api.Comment.update(id, handle=handle, message=comment)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            message = res["comment"]["message"]
            lines = message.split("\n")
            message = "\n".join(["    " + line for line in lines])
            print("id\t\t" + str(res["comment"]["id"]))
            print("url\t\t" + res["comment"]["url"])
            print("resource\t" + res["comment"]["resource"])
            print("handle\t\t" + res["comment"]["handle"])
            print("message\n" + message)
        elif format == "raw":
            print(json.dumps(res))
        else:
            print("id\t\t" + str(res["comment"]["id"]))
            print("url\t\t" + res["comment"]["url"])
            print("resource\t" + res["comment"]["resource"])
            print("handle\t\t" + res["comment"]["handle"])
            print("message\t\t" + res["comment"]["message"].__repr__())

    @classmethod
    def _reply(cls, args):
        api._timeout = args.timeout
        handle = args.handle
        comment = args.comment
        id = args.comment_id
        format = args.format
        if comment is None:
            comment = sys.stdin.read()
        res = api.Comment.create(handle=handle, message=comment, related_event_id=id)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            message = res["comment"]["message"]
            lines = message.split("\n")
            message = "\n".join(["    " + line for line in lines])
            print("id\t\t" + str(res["comment"]["id"]))
            print("url\t\t" + res["comment"]["url"])
            print("resource\t" + res["comment"]["resource"])
            print("handle\t\t" + res["comment"]["handle"])
            print("message\n" + message)
        elif format == "raw":
            print(json.dumps(res))
        else:
            print("id\t\t" + str(res["comment"]["id"]))
            print("url\t\t" + res["comment"]["url"])
            print("resource\t" + res["comment"]["resource"])
            print("handle\t\t" + res["comment"]["handle"])
            print("message\t\t" + res["comment"]["message"].__repr__())

    @classmethod
    def _show(cls, args):
        api._timeout = args.timeout
        id = args.comment_id
        format = args.format
        res = api.Event.get(id)
        report_warnings(res)
        report_errors(res)
        if format == "pretty":
            message = res["event"]["text"]
            lines = message.split("\n")
            message = "\n".join(["    " + line for line in lines])
            print("id\t\t" + str(res["event"]["id"]))
            print("url\t\t" + res["event"]["url"])
            print("resource\t" + res["event"]["resource"])
            print("message\n" + message)
        elif format == "raw":
            print(json.dumps(res))
        else:
            print("id\t\t" + str(res["event"]["id"]))
            print("url\t\t" + res["event"]["url"])
            print("resource\t" + res["event"]["resource"])
            print("message\t\t" + res["event"]["text"].__repr__())



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import (
    GetableAPIResource,
    CreateableAPIResource,
    UpdatableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    ActionAPIResource,
)


class Monitor(
    GetableAPIResource,
    CreateableAPIResource,
    UpdatableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    ActionAPIResource,
):
    """
    A wrapper around Monitor HTTP API.
    """

    _resource_name = "monitor"

    @classmethod
    def get(cls, id, **params):
        """
        Get monitor's details.

        :param id: monitor to retrieve
        :type id: id

        :param group_states: string list indicating what, if any, group states to include
        :type group_states: string list, strings are chosen from one or more \
        from 'all', 'alert', 'warn', or 'no data'

        :returns: Dictionary representing the API's JSON response
        """
        if "group_states" in params and isinstance(params["group_states"], list):
            params["group_states"] = ",".join(params["group_states"])

        return super(Monitor, cls).get(id, **params)

    @classmethod
    def get_all(cls, **params):
        """
        Get all monitor details.

        :param group_states: string list indicating what, if any, group states to include
        :type group_states: string list, strings are chosen from one or more \
        from 'all', 'alert', 'warn', or 'no data'

        :param name: name to filter the list of monitors by
        :type name: string

        :param tags: tags to filter the list of monitors by scope
        :type tags: string list

        :param monitor_tags: list indicating what service and/or custom tags, if any, \
        should be used to filter the list of monitors
        :type monitor_tags: string list

        :returns: Dictionary representing the API's JSON response
        """
        for p in ["group_states", "tags", "monitor_tags"]:
            if p in params and isinstance(params[p], list):
                params[p] = ",".join(params[p])

        return super(Monitor, cls).get_all(**params)

    @classmethod
    def mute(cls, id, **body):
        """
        Mute a monitor.

        :param scope: scope to apply the mute
        :type scope: string

        :param end: timestamp for when the mute should end
        :type end: POSIX timestamp


        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("POST", "mute", id, **body)

    @classmethod
    def unmute(cls, id, **body):
        """
        Unmute a monitor.

        :param scope: scope to apply the unmute
        :type scope: string

        :param all_scopes: if True, clears mute settings for all scopes
        :type all_scopes: boolean

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("POST", "unmute", id, **body)

    @classmethod
    def mute_all(cls):
        """
        Globally mute monitors.

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("POST", "mute_all")

    @classmethod
    def unmute_all(cls):
        """
        Cancel global monitor mute setting (does not remove mute settings for individual monitors).

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("POST", "unmute_all")

    @classmethod
    def search(cls, **params):
        """
        Search monitors.

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("GET", "search", params=params)

    @classmethod
    def search_groups(cls, **params):
        """
        Search monitor groups.

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("GET", "groups/search", params=params)

    @classmethod
    def can_delete(cls, **params):
        """
        Checks if the monitors corresponding to the monitor ids can be deleted.

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("GET", "can_delete", params=params)

    @classmethod
    def validate(cls, **body):
        """
        Checks if the monitors definition is valid.

        :returns: Dictionary representing the API's JSON response
        """
        return super(Monitor, cls)._trigger_class_action("POST", "validate", **body)



# Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2015-Present Datadog, Inc
from datadog.api.resources import DeletableAPISubResource, ListableAPISubResource, AddableAPISubResource


class AwsLogsIntegration(DeletableAPISubResource, ListableAPISubResource, AddableAPISubResource):
    """
    A wrapper around AWS Logs API.
    """

    _resource_name = "integration"
    _resource_id = "aws"

    @classmethod
    def list_log_services(cls, **params):
        """
        List all namespace rules available as options.

        >>> api.AwsLogsIntegration.list_log_services()
        """
        cls._sub_resource_name = "logs/services"
        return super(AwsLogsIntegration, cls).get_items(id=cls._resource_id, **params)

    @classmethod
    def add_log_lambda_arn(cls, **params):
        """
        Attach the Lambda ARN of the Lambda created for the Datadog-AWS \
        log collection to your AWS account ID to enable log collection.

        >>> account_id = "<AWS_ACCOUNT_ID>"
        >>> lambda_arn = "<AWS_LAMBDA_ARN>"

        >>> api.AwsLogsIntegration.add_log_lambda_arn(account_id=account_id, lambda_arn=lambda_arn)
        """
        cls._sub_resource_name = "logs"
        return super(AwsLogsIntegration, cls).add_items(id=cls._resource_id, **params)

    @classmethod
    def save_services(cls, **params):
        """
        Enable Automatic Log collection for your AWS services.

        >>> account_id = "<AWS_ACCOUNT_ID>"
        >>> services = ["s3", "elb", "elbv2", "cloudfront", "redshift", "lambda"]

        >>> api.AwsLogsIntegration.save_services()
        """
        cls._sub_resource_name = "logs/services"
        return super(AwsLogsIntegration, cls).add_items(id=cls._resource_id, **params)

    @classmethod
    def delete_config(cls, **params):
        """
        Delete a Datadog-AWS log collection configuration by removing the specific Lambda ARN \
        associated with a given AWS account.

        >>> account_id = "<AWS_ACCOUNT_ID>"
        >>> lambda_arn = "<AWS_LAMBDA_ARN>"

        >>> api.AwsLogsIntegration.delete_config(account_id=account_id, lambda_arn=lambda_arn)
        """
        cls._sub_resource_name = "logs"
        return super(AwsLogsIntegration, cls).delete_items(id=cls._resource_id, **params)

    @classmethod
    def check_lambda(cls, **params):
        """
        Check function to see if a lambda_arn exists within an account. \
        This sends a job on our side if it does not exist, then immediately returns \
        the status of that job. Subsequent requests will always repeat the above, so this endpoint \
        can be polled intermittently instead of blocking.

        Returns a status of 'created' when it's checking if the Lambda exists in the account.
        Returns a status of 'waiting' while checking.
        Returns a status of 'checked and ok' if the Lambda exists.
        Returns a status of 'error' if the Lambda does not exist.

        >>> account_id = "<AWS_ACCOUNT_ID>"
        >>> lambda_arn = "<AWS_LAMBDA_ARN>"

        >>> api.AwsLogsIntegration.check_lambda(account_id=account_id, lambda_arn=lambda_arn)
        """
        cls._sub_resource_name = "logs/check_async"
        return super(AwsLogsIntegration, cls).add_items(id=cls._resource_id, **params)

    @classmethod
    def check_services(cls, **params):
        """
        Test if permissions are present to add log-forwarding triggers for the \
        given services + AWS account. Input is the same as for save_services.
        Done async, so can be repeatedly polled in a non-blocking fashion until \
        the async request completes

        >>> account_id = "<AWS_ACCOUNT_ID>"
        >>> services = ["s3", "elb", "elbv2", "cloudfront", "redshift", "lambda"]

        >>> api.AwsLogsIntegration.check_services()
        """
        cls._sub_resource_name = "logs/services_async"
        return super(AwsLogsIntegration, cls).add_items(id=cls._resource_id, **params)

    @classmethod
    def list(cls, **params):
        """
        List all Datadog-AWS Logs integrations available in your Datadog organization.

        >>> api.AwsLogsIntegration.list()
        """
        cls._sub_resource_name = "logs"
        return super(AwsLogsIntegration, cls).get_items(id=cls._resource_id, **params)
